<?php

namespace App\Console\Commands;

use Illuminate\Console\Command;
use Illuminate\Support\Facades\Http;

class FetchUsersWithNotStartedCourses extends Command
{
    protected $signature = 'fetch:users-not-started-courses';
    protected $description = 'Fetch users with not started courses from the external API';

    public function __construct()
    {
        parent::__construct();
    }

    public function handle()
    {
        // API URL
        $url = 'https://lmstech.projects-codingbrains.com/api/users-with-not-started-courses';

        try {
            // Make the HTTP request
            $response = Http::get($url);

            // Check if the request was successful
            if ($response->successful()) {
                // Process the response data
                $data = $response->json();
                // Handle the data (e.g., save to database, log, etc.)
                // Example:
                \Log::info('Fetched users with not started courses', $data);
            } else {
                // Log the error or handle it as needed
                \Log::error('Failed to fetch users with not started courses', [
                    'status' => $response->status(),
                    'body' => $response->body(),
                ]);
            }
        } catch (\Exception $e) {
            // Log the exception or handle it as needed
            \Log::error('Error occurred while fetching users with not started courses', [
                'message' => $e->getMessage(),
            ]);
        }
    }
}
