<?php

namespace App\Http\Controllers\Api;

use App\Models\AccountAdmin;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use App\Models\User;

class AccountAdminController extends Controller
{
    // public function assignAdmin(Request $request, $accountId)
    // {
    //     // Validate request
    //      $request->validate([
    //      'admin_id' => 'required|array',
    //      'admin_id.*' => 'exists:users,id',
    //  ]);
    //     // Check if the authenticated user is authorized
    //     $user = Auth::user();
    //     // Add your authorization logic here
    //     $accountAdmin = User::find($accountId);
    //     if ($accountAdmin->role != 1) {
    //         return response()->json(['message' => 'User is not admin'], 400);
    //     }
    //     // Assign admin to the account
    //     foreach ($request->input('admin_id') as $adminId) {
    //     // Check if account_ids is equal to user_id
    //     if ($accountId == $adminId) {
    //         return response()->json([
    //             'message' => 'The main admin and assigned admin cannot be the same person.',
    //         ], 400); // 400 Bad Request
    //     }

    //         $existingAssignment = AccountAdmin::where('account_ids', $accountId)
    //                 ->where('user_id', $adminId)
    //                 ->first();

    //         if ($existingAssignment) {
           
    //         } else {
    //         // If not assigned yet, create the new AccountAdmin record
    //         AccountAdmin::create([
    //         'account_ids' => $accountId,
    //         'user_id' => $adminId,
    //         ]);
            
    //         }
    //     }
    //     return response()->json(['message' => 'Admin assigned to account successfully', 'data' => $accountAdmin], 200);
    // }


    public function assignAdmin(Request $request, $accountId)
    {
        // Validate request
        $request->validate([
            'admin_id' => 'required|array',
            'admin_id.*' => 'exists:users,id',
        ]);
    
        // Check if the authenticated user is authorized
        $user = Auth::user();
    
        // Fetch the account's main admin and validate the role
        $accountAdmin = User::find($accountId);
        if ($accountAdmin->role != 1) {
            return response()->json(['message' => 'User is not an admin'], 400);
        }
    
        $insertedAdmins = [];
        $skippedAdmins = [];
    
        // Loop through all the input admin IDs
        foreach ($request->input('admin_id') as $adminId) {
    
            // Check if the main admin is the same as the assigned admin
            if ($accountId == $adminId) {
                return response()->json([
                    'message' => 'The main admin and assigned admin cannot be the same person.',
                ], 400); // 400 Bad Request
            }
    
            // Check if this admin is already assigned to the account
            $existingAssignment = AccountAdmin::where('account_ids', $accountId)
                                              ->where('user_id', $adminId)
                                              ->first();
    
            if ($existingAssignment) {
                // If admin is already assigned, add to skippedAdmins array
                $skippedAdmins[] = $adminId;
            } else {
                // If not assigned yet, create the new AccountAdmin record
                AccountAdmin::create([
                    'account_ids' => $accountId,
                    'user_id' => $adminId,
                ]);
                // Add to insertedAdmins array
                $insertedAdmins[] = $adminId;
            }
        }
    
        // Prepare response messages based on the results
        if (count($insertedAdmins) > 0 && count($skippedAdmins) > 0) {
            return response()->json([
                'message' => 'Some admins were assigned to the account successfully, but some were already assigned.',
                'inserted' => $insertedAdmins,
                'skipped' => $skippedAdmins,
            ], 200);
        } elseif (count($insertedAdmins) > 0) {
            return response()->json([
                'message' => 'All admins were assigned to the account successfully.',
                'inserted' => $insertedAdmins,
            ], 200);
        } else {
            return response()->json([
                'message' => 'All admins were already assigned to the account.',
                'skipped' => $skippedAdmins,
            ], 400);
        }
    }
    





    

    public function removeAdmin(Request $request, $accountId)
    {
        $request->validate([
            'admin_id' => 'required|array',
            'admin_id.*' => 'exists:users,id',
        ]);

        // Check if the authenticated user is authorized
        $user = Auth::user();
        // Add your authorization logic here

        // Remove admin from the account
        AccountAdmin::where('account_ids', $accountId)
            ->whereIn('user_id', $request->input('admin_id'))
            ->delete();

        return response()->json(['message' => 'Admin removed from account successfully'], 200);
    }



    public function getAssignedAdmins(){
         $users = \DB::table('users')
            ->distinct()
            ->join('account_admin', 'account_admin.account_ids', '=', 'users.id')
            ->select('users.*')
            ->get();

            return response()->json(['users' => $users, 'message' => 'List of main assigned admins']);

    }


    public function getTotalAssignedAccounts(Request $request, $adminId)
    {
        $admin = Auth::user();
        if ($admin->role != 2) {
            return response()->json(['message' => 'User is not admin'], 400);
        }

        $users = \DB::table('users')
        ->join('account_admin', 'users.id', '=', 'account_admin.user_id')
        ->where('account_admin.account_ids', '=', $adminId)
        ->select('users.*')
        ->get();


        return response()->json([
            'users' => $users, 
            'message' => 'List of main assigned users'
        ]);
    }

}
