<?php

namespace App\Http\Controllers\Api;

use Carbon\Carbon;
use App\Models\EmailVerification;
use Illuminate\Support\Facades\Mail;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Hash;
use App\Http\Requests\LoginRequest;
use App\Http\Requests\SignupRequest;
use App\Models\User;
use http\Env\Response;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Validation\Rules\Password;
use App\Models\QuickLink;
use App\Models\ManageQuickLink;
use App\Models\AccountAdmin;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\ValidationException;


class AuthController extends Controller

{

        public function signup(SignupRequest $request, User $user)
    {
        try {
            $data = $request->validated();
            $data['role'] = 0;
            $user = $user->create([
                'name' => $data['name'],
                'email' => $data['email'],
                'password' => bcrypt($data['password']),
                'role' => $data['role'],
            ]);

            $token = $user->createToken('user')->plainTextToken;
            $response = [
                'user' => $user,
                'token' => $token,
            ];

            return response()->json($response, 201);
        } catch (\Exception $e) {
            \Log::error('Error creating user: ' . $e->getMessage());
            return response()->json(['error' => 'Internal Server Error'], 500);
        }
    }

    public function logout(Request $request)

    {

        /** @var \App\Models\User $user */

        $user = $request->user();

        $user->currentAccessToken()->delete();

        return response('', 200);

    }


// mail functionality


   public function sendOtp(User $user)
    {
        $otp = rand(100000, 999999);
        EmailVerification::updateOrCreate(
            ['email' => $user->email],
            [
                'email' => $user->email,
                'otp' => $otp,
                'created_at' => Carbon::now(),
            ]
        );

        $data['email'] = $user->email;
        $data['title'] = 'Mail Verification';
        $data['body'] = 'Your OTP is: ' . $otp;

        try {
            Mail::send('mailVerification', ['data' => $data], function ($message) use ($data) {
                $message->to($data['email'])->subject($data['title']);
            });

            return response()->json(['message' => 'OTP sent successfully']);
        } catch (\Exception $e) {
            return response()->json(['message' => 'Failed to send OTP. Please try again.'], 500);
        }
    }

    public function requestOtp(Request $request)
    {
        $request->validate([
            'email' => 'required|email',
        ]);

        $user = User::where('email', $request->email)->first();

        if ($user) {
            $this->sendOtp($user);

            return response()->json(['message' => 'OTP sent successfully']);
        } else {
            return response()->json(['message' => 'User not found'], 404);
        }
    }

    public function verifyOtp(Request $request, User $user)
    {
        $request->validate([
            'email' => 'required|email',
            'otp' => 'required|digits:6',
        ]);

        if (!$user) {
            return response()->json(['message' => 'User not found'], 422);
        }

        $otpData = EmailVerification::where('email', $request->email)->where('otp', $request->otp)->first();

        if ($otpData) {
            $currentTime = Carbon::now();
            $time = $otpData->created_at;

            if ($currentTime >= $time && $time >= $currentTime->subSeconds(90)) {
                User::where('id', $user->id)->update(['is_verified' => 1]);

                return response()->json(['message' => 'Mail has been verified']);
            } else {
                return response()->json(['message' => 'Your OTP has expired'], 422);
            }
        } else {
            return response()->json(['message' => 'You entered the wrong OTP'], 422);
        }
    }


//google login
 public function googleLogin(Request $request)
    {
        
        $data = $request->validate([
            'name' => 'required|string',
            'email' => 'required|string',
            'google_secret_key' => 'nullable|string'
        ]);

        try {
            $user = User::where('email', $data['email'])->first();
			// $user = Auth::user();
            if ($user) {
                
                $user->update([
                    'name' => $data['name'],
                    'google_secret_key' => $data['google_secret_key'],
                    'user_type' => 1
                ]);
            } else {
              
                $user = User::create([
                    'name' => $data['name'],
                    'email' => $data['email'],
                    'google_secret_key' => $data['google_secret_key'],
                    'user_type' => 1
                ]);
            }
            $token = $user->createToken('user')->plainTextToken;
            \Log::info('User created successfully: ' . $user->id);

            return response()->json([
                'data' => [
                    'message' => $user ? 'User updated successfully' : 'User created successfully',
                    'user' => $user,
                    'token' => $token, 
                ]
            ], 201);
        } catch (\Exception $e) {
            \Log::error('Error creating user: ' . $e->getMessage());
            return response()->json(['error' => 'Internal Server Error'], 500);
        }
    }


// 2FA Enable Disable Functionality

    public function enableTwoFa(Request $request)
    {
       
        if ($request->user()) {
            $user = $request->user();

            $user->update(['mfa' => true]);

            return response()->json(['message' => 'Two-Factor Authentication enabled successfully']);
        } else {
            return response()->json(['message' => 'User not authenticated'], 401);
        }
    }


    public function disableTwoFa(Request $request)
    {
        if ($request->user()) {
            $user = $request->user();
            $user->update(['mfa' => false]);
            return response()->json(['message' => 'Two-Factor Authentication disabled successfully']);
        } else {
            return response()->json(['message' => 'User not authenticated'], 401);
        }
    }

        public function changePassword(Request $request)
    {
        $data = $request->validate([
                'password' => [
                'required',
                Password::min(8)
                    ->letters()
                    ->symbols()
                    ->numbers()
            ],
            'confirm_password' => 'required|same:password',
        ]);

        if ($request->user()) {
            $user = $request->user();
            $user->update([
                'password' => Hash::make($data['password']),
            ]);

            return response()->json(['message' => 'Password changed successfully']);
        } else {
            return response()->json(['message' => 'User not found']);
        }
    }

     public function forgetPassword(Request $request)
    {
        $request->validate([
            'email' => 'required|email',
            'old_password' => 'required',
            'password' => 'required|min:8|confirmed',
        ]);


        $user = User::where('email', $request->input('email'))->first();

        if (!$user) {
            return response()->json(['error' => 'User not found'], 404);
        }


        if (!Hash::check($request->input('old_password'), $user->password)) {
            return response()->json(['error' => 'Old password does not match'], 401);
        }


        $user->update([
            'password' => Hash::make($request->input('password')),
        ]);

        return response()->json(['message' => 'Password changed successfully']);
    }


// 2fa enabled user logic goes here.. 

        protected function generateLoginOTP($email)
    {
        $otp = rand(100000, 999999);

        EmailVerification::updateOrCreate(
            ['email' => $email],
            [
                'email' => $email,
                'otp' => $otp,
                'created_at' => now(),
            ]
        );

        $this->sendOtpEmail($email, $otp);

        return $otp;
    }

    protected function sendOtpEmail($email, $otp)
    {
        $data = [
            'email' => $email,
            'title' => '2FA Otp Verification',
            'body' => 'Your OTP is: ' . $otp,
        ];

        try {
            Mail::send('passwordReset', ['data' => $data], function ($message) use ($data) {
                $message->to($data['email'])->subject($data['title']);
            });

            
            \Log::info('OTP sent successfully for user: ' . $email);
        } catch (\Exception $e) {
           
            \Log::error('Failed to send OTP for user: ' . $email . '. Error: ' . $e->getMessage());
        }
    }

    public function verifyLoginOtp(Request $request)
    {
         $messages = [
            'otp.digits' => 'The otp must be exactly 6 digits.',
            'email.required' => 'The email field is required.',
            'email.email' => 'Please provide a valid email address.',
        ];

        try {
            $request->validate([
                'email' => 'required|email',
                'otp' => 'required|digits:6',
            ], $messages);

        } catch (ValidationException $e) {
            $firstErrorMessage = collect($e->errors())->flatten()->first();
            
            return response()->json([
                'message' => $firstErrorMessage, 
                'errors' => $e->errors(),
            ], 422);
        }

        $user = User::where('email', $request->email)->first();

        if (!$user) {
            return response()->json(['message' => 'User not found'], 422);
        }

        $otpData = EmailVerification::where('email', $request->email)
            ->where('otp', $request->otp)
            ->first();

        if (!$otpData) {
            return response()->json(['message' => 'Invalid OTP. Please try again.'], 401);
        }

        $otpData->delete();

        $tokenName = $user->role === 1 ? 'admin' : 'user';
        $token = $user->createToken($tokenName)->plainTextToken;

        $response = [
            'user' => $user,
            'token' => $token,
            'role' => $user->role,
        ];

        return response()->json($response, 201);
    }

    public function login(Request $request)
    {
        // dd($request);

            // Custom error messages
            $messages = [
                'email.required' => 'Please enter your email address.',
                'email.email' => 'The email format is invalid. Please enter a valid email.',
                'email.exists' => 'This email is not registered in our system. Please check and try again.',
                'password.required' => 'The password field is required.',
                'remember.boolean' => 'The remember me field must be true or false.',
            ];
        
            try {
                // Validate the request with custom messages
            $credentials=$request->validate([
                    'email' => 'required|email|string|exists:users,email',
                    'password' => 'required',
                    'remember' => 'boolean',
                ], $messages);
        
                // Perform login logic here...
        
            } catch (ValidationException $e) {
                $firstErrorMessage = collect($e->errors())->flatten()->first();
                return response()->json([
                    'message' => $firstErrorMessage,
                    'errors' => $e->errors(),
                ], 422);
            }

        // $credentials = $request->validated();

        if (!Auth::attempt($credentials)) {
            return response([
                'message' => 'Provided email or password is incorrect'
            ], 400);
        }

        /** @var \App\Models\User $user */
        $user = Auth::user();
        $accountadmins = $user->accountadmins()->get();

        \Log::info('User MFA value: ' . $user->mfa);

        if ($user->mfa) {
           
            \Log::info('Two-Factor Authentication enabled. Asking for OTP.');

            $mfastatus = $user->mfa;
            $otp = $this->generateLoginOTP($user->email);

           
            return response()->json(['message' => 'Two-Factor Authentication enabled. Please provide OTP for verification.', 'mfa' => $mfastatus]);
        }

        // Directly log in the user
        \Log::info('Two-Factor Authentication not enabled. Directly logging in.');
        $tokenName = $user->role === 1 ? 'admin' : 'user';
        $token = $user->createToken($tokenName)->plainTextToken;

        $userId = $user->id;
         Log::info('User found', ['user_id' => $userId]);
        $multiple = AccountAdmin::where('user_id', $userId)->first();
        if ($multiple) {
            $adminid = $multiple->account_ids;
            $admin = User::find($adminid);
            // return response([
            //     'message' => 'Admin details found successfully'
            // ], 200);
        } else {
            $admin = NULL;
            // return response([
            //     'message' => 'Admin details not found'
            // ], 400);
        }

        $response = [
            'user' => $user,
            'token' => $token,
            'role' => $user->role,
            'admin' => $admin,
            'message'=>'Employee Loggedin Successfully!!',
        ];

        return response()->json($response, 201);
    }


    // public function createAdmin(SignupRequest $request, User $admin)
    // {
    //     try {
    //         $data = $request->validated();

    //         $authenticatedUser = Auth::user();

    //         if (!$authenticatedUser || $authenticatedUser->role !== 2) {
    //             return response()->json(['error' => 'Unauthorized, only super admin can create Admins'], 401);
    //         }

    //         $data['role'] = 1;

    //         // Create the admin
    //         $newAdmin = $admin->create([
    //             'name' => $data['name'],
    //             'email' => $data['email'],
    //             'company_name' => $data['company_name'],
    //             'password' => bcrypt($data['password']),
    //             'role' => $data['role'],
    //         ]);

    //         // Log successful admin creation
    //         Log::info('Admin created successfully: ' . $newAdmin->id);

    //         // Fetching default link IDs based on link names
    //         $defaultLinkNames = ['Admin Dashboard', 'Employees', 'Resources', 'Policies', 'Terms & Conditions'];
    //         $defaultLinkIds = QuickLink::whereIn('link_title', $defaultLinkNames)->pluck('id')->toArray();

    //         // Log default link IDs
    //         Log::info('Default link IDs: ' . implode(', ', $defaultLinkIds));

    //         // Assign default links to the admin
    //         foreach ($defaultLinkIds as $linkId) {
    //             $existingAssignment = ManageQuickLink::where('admin_id', $newAdmin->id)
    //                 ->where('link_id', $linkId)
    //                 ->exists();

    //             if (!$existingAssignment) {
    //                 $manageQuickLink = new ManageQuickLink();
    //                 $manageQuickLink->admin_id = $newAdmin->id;
    //                 $manageQuickLink->link_id = $linkId;
    //                 $manageQuickLink->save();

    //                 // Log link assignment
    //                 Log::info('Link ID ' . $linkId . ' assigned to admin: ' . $newAdmin->id);
    //             }
    //         }

    //         // Create token for the admin
    //         $token = $newAdmin->createToken('user')->plainTextToken;

    //         // Prepare response
    //         $response = [
    //             'Admin' => $newAdmin,
    //             'token' => $token,
    //             'message' => 'Admin created successfully with default links assigned.',
    //         ];

    //         return response()->json($response, 201);
    //     } catch (\Exception $e) {
    //         // Log error
    //         Log::error('Error creating user: ' . $e->getMessage());

    //         return response()->json(['error' => 'Internal Server Error'], 500);
    //     }
    // }


    public function createAdmin(Request $request, User $admin)
    {
        try {
            // Manually create the validator instance
            $validator = Validator::make($request->all(), [
                'name' => 'required|string|max:255',
                'email' => 'required|email|unique:users,email',
                'company_name' => 'required|string|max:255',
                'password' => 'required|string|min:6',
            ], [
                'name.required' => 'The name field is required.',
                'email.required' => 'The email field is required.',
                'email.email' => 'The email must be a valid email address.',
                'email.unique' => 'The email has already been taken.',
                'company_name.required' => 'The company name is required.',
                'password.required' => 'The password is required.',
                'password.min' => 'The password must be at least 6 characters.',
            ]);
    
            // If validation fails, return the error message
            if ($validator->fails()) {
                return response()->json([
                    'message' => $validator->errors()->first(),
                    'errors' => $validator->errors(),
                ], 422);
            }
    
            // Check if the email already exists in the database
            $existingUser = User::where('email', $request->input('email'))->first();
    
            if ($existingUser) {
                return response()->json([
                    'message' => 'The email has already been taken.',
                    'errors' => ['email' => ['The email has already been taken.']],
                ], 422);
            }
    
            // Proceed with creating a new admin
            $authenticatedUser = Auth::user();
    
            if (!$authenticatedUser || $authenticatedUser->role !== 2) {
                return response()->json(['error' => 'Unauthorized, only super admin can create Admins'], 401);
            }
    
            $data = $validator->validated();
            $data['role'] = 1;
    
            // Create the admin
            $newAdmin = $admin->create([
                'name' => $data['name'],
                'email' => $data['email'],
                'company_name' => $data['company_name'],
                'password' => bcrypt($data['password']),
                'role' => $data['role'],
            ]);
    
            // Assign default links and handle the rest of the logic...
    
            // Create token for the admin
            $token = $newAdmin->createToken('user')->plainTextToken;
    
            // Prepare response
            $response = [
                'Admin' => $newAdmin,
                'token' => $token,
                'message' => 'Admin created successfully with default links assigned.',
            ];
    
            return response()->json($response, 201);
    
        } catch (\Exception $e) {
            Log::error('Error creating user: ' . $e->getMessage());
    
            return response()->json(['error' => 'Internal Server Error'], 500);
        }
    }
    
    
}

