<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\EmailTemplate;
use App\Services\EmailService;
use App\Models\TemplateCategory;
use App\Models\User;
use Illuminate\Support\Facades\Auth;
use Illuminate\Validation\Rule;
use App\Helpers\AdminHelper;

class EmailTemplateController extends Controller
{
   
public function index($templateCategoryId = null)
{

    $adminId = auth()->user()->id; 
     $admin = Auth::user();
        // $adminData = AdminHelper::getAdminAccount($admin);

        // if ($adminData) {
        //         $adminId = $adminData ? $adminData->id : $adminId;
        // }

        $assignedId = AdminHelper::getAssignedId();

        // Determine the admin ID based on the value of assignedId
        $adminId = ($assignedId === "null") ? $admin->id : $assignedId; 


    // Use where clause to filter templates by category and admin_id
    $templates = $templateCategoryId
        ? EmailTemplate::where('template_category_id', $templateCategoryId)
            ->where('admin_id', $adminId)
            ->get()
        : EmailTemplate::where('admin_id', $adminId)->get();

    // dd($templates);

    
    $templatesCount = $templates->count();

 
    $categoryIdsWithTemplates = $templates->pluck('template_category_id')->unique()->toArray();

   
    $categories = TemplateCategory::whereIn('id', $categoryIdsWithTemplates)->get();

    foreach ($templates as $template) {
        $category = $categories->where('id', $template->template_category_id)->first();
        $template->category_name = $category ? $category->name : null;
    }
    return response()->json(['data' => $templates, 'count' => $templatesCount]);
}

public function singletemplate($id)
{

    $templates = EmailTemplate::where('id', $id)->first();
    
    return response()->json(['data' => $templates]);
}

    public function store(Request $request)
    {
        $request->validate([
        'name' => 'required|string|max:255',
        'content' => 'required|string',
        'sender_name' => 'nullable|string|max:255',
        'sender_address' => 'nullable|email',
        'subject' => 'nullable|string|max:255',
        'template_category_id' => 'nullable|exists:template_categories,id',
        'landing_page_id' => 'nullable|exists:landing_pages,id',
        'rating' => 'nullable',
    ]);
        $admin = Auth::user();

        $assignedId = AdminHelper::getAssignedIdByPost($request);

        // Determine the admin ID based on the value of assignedId
        $adminId = ($assignedId === "null") ? $admin->id : $assignedId; 

        
        //     // Concatenate {{open_tracking_pixel}} in the content field
    $request->merge(['content' => $request->input('content') . '{{open_tracking_pixel}}']);

        try {
            // Ensure the user is authenticated
            if (Auth::check()) {
                // Get the authenticated user's ID
                // $adminId = Auth::id();

                // Merge the admin_id into the request data
                $requestData = array_merge($request->only(['name', 'content', 'sender_name', 'sender_address', 'subject', 'template_category_id', 'landing_page_id','rating']), ['admin_id' => $adminId]);

                $existingtemplate = EmailTemplate::where('name', $request->name)
                                        ->where('admin_id', '=', $adminId)
                                        ->first();
                if($existingtemplate){
                    return response()->json(['message'=>'Template name already exists!'], 400);
                }

                // Create the email template
                $template = EmailTemplate::create($requestData);

                return response()->json(['message'=>'Template Created Successfully!!',$template, 201]);
            } else {
                // User is not authenticated
                return response()->json(['error' => 'Unauthorized'], 401);
            }
        } catch (\Exception $e) {
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $id)
    {
        $adminId = Auth::id();
        $request->validate([
            'name' => 'required|string|max:255',
            'content' => 'required|string',
            'sender_name' => 'nullable|string|max:255',
            'sender_address' => 'nullable|email',
            'subject' => 'nullable|string|max:255',
            'template_category_id' => 'nullable|exists:template_categories,id',
            'landing_page_id' => 'nullable|exists:landing_pages,id',
            'rating'=> 'nullable',
        ]);

        $template = EmailTemplate::findOrFail($id);
        $existingtemplate = EmailTemplate::where('name', $request->name)
                                        ->where('id', '!=', $id)
                                        ->where('admin_id', '=', $adminId)
                                        ->first();
        if($existingtemplate){
            return response()->json(['message'=>'Template name already exists!'], 400);
        }else{
            $template->update($request->only(['name', 'content', 'sender_name','sender_address','subject','template_category_id', 'landing_page_id','rating']));
            return response()->json(['message'=>'Template Updated Successfully!!', 'data'=>$template ]);
        }
    }

    public function destroy($id)
    {
        $template = EmailTemplate::findOrFail($id);
        $template->delete();

        return response()->json(['message' => 'Template deleted successfully']);
    }

    public function sendEmail($templateId, $userId, Request $request)
    {
        try {
            $admin = Auth::user();
            $template = EmailTemplate::findOrFail($templateId);
            $user = User::findOrFail($userId);
            $to = $user->email;

            $data = [
                'name' => $user->name,
                'sender_name' => $request->input('sender_name', $template->sender_name),
                'sender_address' => $request->input('sender_address', $template->sender_address),
                'subject' => $request->input('subject', $template->subject),
               
            ];
            $emailService = new EmailService(); 

            $emailserviceresponse =  $emailService->sendEmail($template->id, $to, $admin->id, ['user_id' => $user->id] + $data);

            return response()->json(['message' => 'Email sent successfully', 'emailresponse' => $emailserviceresponse, 'to' => $to, 'data' => $data]);
        } catch (\Exception $e) {
            return response()->json(['error' => 'Error sending email', 'message' => $e->getMessage()], 500);
        }
    }

    public function sendEmailToMultipleUsers(EmailService $emailService, $templateId, Request $request)
    {
        
        try {
            $template = EmailTemplate::findOrFail($templateId);
            $admin = Auth::user();
            $data = $request->validate([
                    'user_ids' => 'required|array',
                    'user_ids.*' => 'exists:users,id',
                    'sender_name' => 'string',
                    'sender_address' => 'email',
                    'subject' => 'string',
                ]);

            $users = User::find($data['user_ids']);

            foreach ($users as $user) {
                $userData = [
                    'name' => $user->name,
                    'sender_name' => $data['sender_name'] ?? $template->sender_name,
                    'sender_address' => $data['sender_address'] ?? $template->sender_address,
                    'subject' => $data['subject'] ?? $template->subject,
                ];

             
                $emailService->sendEmail($template->id, $user->email, $admin->id, ['user_id' => $user->id] + $userData);
            }

            return response()->json(['message' => 'Email sent successfully']);
        } catch (\Exception $e) {
            return response()->json(['error' => 'Error sending email', 'message' => $e->getMessage()], 500);
        }
    }
}
