<?php

namespace App\Http\Controllers\Api;

use App\Models\File;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use App\Models\User;
use Carbon\Carbon;
use App\Helpers\AdminHelper;
class FileUploadController extends Controller
{


public function uploadFile(Request $request)
{
    $admin = Auth::user();
    if ($admin->role !== 1 && $admin->role !== 2) {
        return response()->json(['message' => 'Unauthorized. Only admins can upload the content.'], 403);
    }
    
    // Ensure files are present in the request and is an array
    if (!$request->hasFile('files') || !is_array($request->file('files'))) {
        return response()->json(['message' => 'No files were provided or files are not in the correct format.'], 400);
    }
    
    $request->validate([
        'files.*' => 'required|file|mimes:jpg,png,pdf,jpeg,docx,svg,txt|max:16384',
    ]);
    
    $uploadedFiles = [];
    
    foreach ($request->file('files') as $file) {
        $fileName = time() . '_' . $file->getClientOriginalName();
        $file->move(public_path('uploads'), $fileName);
        $data = [
            'path' => 'uploads/' . $fileName,
            'filename' => $fileName,
            'admin_id' => $admin->id,
        ];

            $admin = Auth::user();
            $AdminSwitchId=$request->input('AdminSwitchId');

            $adminId = ($AdminSwitchId === "null") ? $admin->id : $AdminSwitchId; 
            if ($adminId) {
                $data['admin_id'] = $adminId;
            }

        $uploadedFile = File::create($data);
        $uploadedFiles[] = $uploadedFile;
    }
    
    $responseMessage = count($uploadedFiles) === 1 ? 'File uploaded successfully' : 'Files uploaded successfully';

    return response()->json(['message' => $responseMessage, 'files' => $uploadedFiles]);
}



public function getFiles()
{
    $user = Auth::user();
    $userId = $user->id;

    // Check if the user is a super admin
    if ($user->role === 2) {
        // For super admins with role 2, get all files uploaded by all admins
        $files = File::orderByDesc('created_at')->get();
    } elseif ($user->role === 1) {
        // For admins with role 1, get files uploaded by themselves and their associated users
        // $files = File::where('admin_id', $userId)->orderByDesc('created_at')->get();
        $admin = Auth::user();
        $assignedId = AdminHelper::getAssignedId();
        $adminId = ($assignedId === "null") ? $admin->id : $assignedId;
        // print_r($adminId); die;
        $files = File::where('admin_id', $adminId)->orderByDesc('created_at')->get();
        //$files = $files->merge(File::where('admin_id', $assignedId)->orderByDesc('created_at')->get());
    } else {
        // For non-admin users, get files uploaded by their associated admin
        $files = File::where('admin_id', $user->admin_id)->orderByDesc('created_at')->get();
    }

    $formattedFiles = $files->map(function ($file) {
        return [
            'id' => $file->id,
            // 'filename' => $file->filename,
            'filename' => substr($file->filename, 11),
            'path' => $file->path,
            'created_at' => $file->created_at ? Carbon::parse($file->created_at)->format('m/d/Y') : null,
            'admin_id' => $file->admin_id,
        ];
    });

    return response()->json(['files' => $formattedFiles]);
}


     // Edit file details
    public function editFile(Request $request, $id)
    {
        $request->validate([
            'filename' => 'required|string|max:255',
        ]);

        $file = File::find($id);

        if (!$file) {
            return response()->json(['error' => 'File not found'], 404);
        }

        $file->update(['filename' => $request->input('filename')]);

        return response()->json(['message' => 'File updated successfully', 'file' => $file]);
    }

    // Delete file
    public function deleteFile($id)
    {
        $file = File::find($id);

        if (!$file) {
            return response()->json(['error' => 'File not found'], 404);
        }

        // Delete the file from storage
        $filePath = public_path($file->path);
        if (file_exists($filePath)) {
            unlink($filePath);
        }

        // Delete the file record from the database
        $file->delete();

        return response()->json(['message' => 'File deleted successfully']);
    }

    public function deleteFiles(Request $request)
{
    $fileIds = $request->input('file_ids');

    if (empty($fileIds)) {
        return response()->json(['error' => 'No file IDs provided'], 400);
    }

    foreach ($fileIds as $id) {
        $file = File::find($id);

        if (!$file) {
            return response()->json(['error' => "File with ID $id not found"], 404);
        }

        // Delete the file from storage
        $filePath = public_path($file->path);
        if (file_exists($filePath)) {
            unlink($filePath);
        }

        // Delete the file record from the database
        $file->delete();
    }

    return response()->json(['message' => 'Files deleted successfully']);
}

}
