<?php

namespace App\Http\Controllers\Api;

use App\Models\User;
use App\Models\EmailVerification;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\Validator;
use App\Http\Controllers\Controller;
use Illuminate\Validation\Rules\Password;
class ForgotPasswordController extends Controller
{


public function sendOtp(Request $request)
{
    $validator = Validator::make($request->all(), [
        'email' => 'required|email|exists:users,email',
    ], [
        'email.exists' => 'User not found. Please enter a valid email address.',
    ]);

    if ($validator->fails()) {
        return response()->json(['message' => $validator->errors()->first()], 422);
    }

    $user = User::where('email', $request->email)->first();

    // Check if the user is already verified
    $verification = EmailVerification::where('email', $user->email)->first();
    if ($verification && $verification->is_verified) {
        return response()->json(['message' => 'User is already verified.'], 422);
    }

    try {
        // Validate email existence
        if (!$user) {
            throw new \Exception('User not found');
        }

        $otp = rand(100000, 999999);

        // Update or create EmailVerification record
        EmailVerification::updateOrCreate(
            ['email' => $user->email],
            [
                'email' => $user->email,
                'otp' => $otp,
                'is_verified' => false, // Set as false for a new OTP
                'created_at' => Carbon::now(),
            ]
        );

        $data['email'] = $user->email;
        $data['title'] = 'Password Reset';
        $data['body'] = 'Your OTP for password reset is: ' . $otp;

        // Send email
        Mail::send('passwordReset', ['data' => $data], function ($message) use ($data) {
            $message->to($data['email'])->subject($data['title']);
        });

        \Log::info('OTP sent successfully for user: ' . $user->email);
        return response()->json(['message' => 'OTP sent successfully']);
    } catch (\Exception $e) {
        $errorMessage = $e->getMessage();

        if ($errorMessage === 'User not found') {
            return response()->json(['message' => 'User not found. Please enter a valid email address.'], 422);
        }

        \Log::error('Failed to send OTP for user: ' . $user->email . '. Error: ' . $errorMessage);
        return response()->json(['message' => 'Failed to send OTP. Please try again.'], 500);
    }
}

    public function verifyOtpAndReset(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'email' => 'required|email|exists:users,email',
            'otp' => 'required|digits:6',
            'new_password' => ['required',Password::min(8)->letters()->symbols()->numbers()],
        ]);

        if ($validator->fails()) {
            return response()->json(['message' => $validator->errors()->first()], 422);
        }

        $user = User::where('email', $request->email)->first();

        $otpData = EmailVerification::where('email', $request->email)
            ->where('otp', $request->otp)
            ->first();

        if ($otpData) {
            $currentTime = now();
            $expiryTime = Carbon::parse($otpData->created_at)->addMinutes(5);

            if ($currentTime <= $expiryTime) {
                $user->update(['password' => bcrypt($request->new_password)]);
                $otpData->delete();
                return response()->json(['message' => 'Password reset successfully']);
            } else {
                return response()->json(['message' => 'Password reset OTP has expired'], 422);
            }
        } else {
            return response()->json(['message' => 'Invalid OTP'], 422);
        }
    }
}
