<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\LandingPageCategory;
use Illuminate\Support\Facades\Auth;
use App\Helpers\AdminHelper;

class LandingPageCategoryController extends Controller
{
    public function index()
    {

    	 $adminId = Auth::user()->id;
         $admin = Auth::user();

         $assignedId = AdminHelper::getAssignedId();

         $adminId = ($assignedId === "null") ? $admin->id : $assignedId; 

        // $adminData = AdminHelper::getAdminAccount($admin);
        // if ($adminData){
        //     $adminId = $adminData ? $adminData->id : $adminId ;
        // }
    	 
        // Get all template categories with the count of associated templates
        $categories = LandingPageCategory::where('admin_id', $adminId)
        ->withCount('landingPages')->get();

        // Format the response
        $formattedCategories = $categories->map(function ($category) {
            return [
                'id' => $category->id,
                'name' => $category->name,
                'count' => $category->landing_pages_count,
            ];
        });

        return response()->json(['data' => $formattedCategories]);
    }



    public function store(Request $request)
    {
        // $admin = Auth::user();
        // $adminId = Auth::user()->id;
        // $adminData = AdminHelper::getAdminAccount($admin);
        // if ($adminData){
        //     $adminId = $adminData ? $adminData->id : $adminId ;
        // }

        $admin = Auth::user();

        $assignedId = AdminHelper::getAssignedIdByPost($request);

        // Determine the admin ID based on the value of assignedId
        $adminId = ($assignedId === "null") ? $admin->id : $assignedId; 


        $request->validate([
            'name' => 'required|string|max:255',
        ], [
            'name.required' => 'Please enter Landing Category name',
        ]);

        $categoryData = $request->only('name');
        $categoryData['admin_id'] = $adminId;

        if($admin->createdLandingPageCategory()->where('name', $categoryData['name'])->where('admin_id', $adminId)->exists()){
            return response()->json(['message'=>'Sorry admin, you have already created the landing category with same name please choose another name'],403);
        }

        $category = LandingPageCategory::create($categoryData);

        return response()->json(['message'=>'Landing Category added successfully!!', 'category' => $category], 201);
    }


    public function update(Request $request, $categoryId)
    {
        // $admin = Auth::user();
        // $adminId = Auth::user()->id;
        // $adminData = AdminHelper::getAdminAccount($admin);
        // if ($adminData){
        //     $adminId = $adminData ? $adminData->id : $adminId ;
        // }

        $admin = Auth::user();

        $assignedId = AdminHelper::getAssignedIdByPost($request);

        // Determine the admin ID based on the value of assignedId
        $adminId = ($assignedId === "null") ? $admin->id : $assignedId; 

        $request->validate([
            'name' => 'required|string|max:255',
        ]);

        $category = LandingPageCategory::find($categoryId);
        if(!$category){
            return response()->json(['message'=>'Category Not Found!!'],404);
        }

    $existingCategory = LandingPageCategory::where('name', $request->name)
                                        ->where('id', '!=', $categoryId)
                                        ->where('admin_id', '=', $adminId)
                                        ->first();


    if($existingCategory) {
        return response()->json(['message' => 'Category name already exists!'], 400);
    }
        $category->update($request->only('name'));

        return response()->json(['message'=>'Landing Category updated successfully!!','category' => $category], 200);
    }

    public function destroy($categoryId)
    {
        $category = LandingPageCategory::find($categoryId);
        if(!$category){
            return response()->json(['message'=>'Category Not Found!!'],404);
        }
        $category->delete();

        return response()->json(['message' => 'Landing Category deleted successfully!!'], 200);
    }
}
