<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use App\Models\LandingPage;
use App\Models\LandingPageCategory;
use App\Helpers\AdminHelper;

class LandingPageController extends Controller
{


    public function index($pageCategoryId = null)
    {

        $adminId = auth()->user()->id;
        $admin = Auth::user();

        $assignedId = AdminHelper::getAssignedId();

        // Determine the admin ID based on the value of assignedId
        $adminId = ($assignedId === "null") ? $admin->id : $assignedId; 

        // $adminData = AdminHelper::getAdminAccount($admin);
        // if ($adminData){
        //     $adminId = $adminData ? $adminData->id : $adminId ;
        // }

        $pages = $pageCategoryId
            ? LandingPage::where('landing_page_category_id', $pageCategoryId)
            ->where('admin_id', $adminId)
            ->get()
            : LandingPage::where(
                'admin_id',
                // $adminData ? $adminData->id : $adminId
                $adminId
            )->orderbyDesc('id')->get();

        $pageCount = $pages->count();

        $categoryIdsWithpages = $pages->pluck('landing_page_category_id')->unique()->toArray();

        $categories = LandingPageCategory::whereIn('id', $categoryIdsWithpages)->get();

            foreach ($pages as $page) {
        $category = $categories->where('id', $page->landing_page_category_id)->first();
        $page->category_name = $category ? $category->name : null;
    }

        return response()->json(['data' => $pages, 'count' => $pageCount]);
    }



    public function store(Request $request)
    {
        $request->validate([
            'page_title' => 'required|string|max:255',
            'page_content' => 'required|string',
            'landing_page_category_id' => 'required|exists:landing_page_categories,id',
        ]);
        // $admin = Auth::user();

        // $adminData = AdminHelper::getAdminAccount($admin);

        $admin = Auth::user();

        $assignedId = AdminHelper::getAssignedIdByPost($request);

        // Determine the admin ID based on the value of assignedId
        $adminId = ($assignedId === "null") ? $admin->id : $assignedId; 


        if ($admin->role !== 1 && $admin->role !== 2) {
            return response()->json(['message' => 'Only admins can create Landing Pages']);
        }

        $existingpage = LandingPage::where('page_title', $request->page_title)
                                        ->where('admin_id', '=', $adminId)
                                        ->first();
        if($existingpage){
            return response()->json(['message'=>'Page title already exists!'], 400);
        }

        $landingPage = LandingPage::create([
            'page_title' => $request->input('page_title'),
            'page_content' => $request->input('page_content'),
            'admin_id' => $adminId,
            'landing_page_category_id' => $request->input('landing_page_category_id'),
        ]);

        return response()->json(['data' => $landingPage], 201);
    }


   public function singlePage($id)
{

    $landingpage = LandingPage::where('id', $id)->first();
    

    return response()->json(['data' => $landingpage]);
}

    public function update(Request $request, $id)
    {
        $admin = Auth::user();
        $request->validate([
            'page_title' => 'required|string|max:255',
            'page_content' => 'required|string',
            'landing_page_category_id' => 'required|exists:landing_page_categories,id',
        ]);

        $page = LandingPage::findOrFail($id);
        $existingpage = LandingPage::where('page_title', $request->page_title)
                                        ->where('id', '!=', $id)
                                        ->where('admin_id', '=', $admin->id)
                                        ->first();
        if($existingpage){
            // dd($existingpage);
            return response()->json(['message'=>'Page title already exists!'], 400);
        }else{
            $page->update($request->only(['page_title', 'page_content', 'landing_page_category_id']));
            return response()->json(['message' => 'Template updated successfully', 'data' => $page], 200);
        }
    }

    public function destroy($id)
    {
        // Get the authenticated admin user
        $admin = Auth::user();

        $landingPage = $admin->landingPages()->find($id);

        if (!$landingPage) {
            return response()->json(['error' => 'Landing page not found'], 404);
        }

        $landingPage->delete();

        return response()->json(['message' => 'Landing page deleted successfully'], 200);
    }
}
