<?php

namespace App\Http\Controllers;

use App\Models\User;
use App\Models\EmailVerification;
use Illuminate\Support\Facades\Mail;
use Illuminate\Http\Request;

class MailController extends Controller
{
    public function sendOtp($user)
    {
        $otp = rand(100000, 999999);
        $time = time();

        EmailVerification::updateOrCreate(
            ['email' => $user->email],
            [
                'email' => $user->email,
                'otp' => $otp,
                'created_at' => $time,
            ]
        );

        $data['email'] = $user->email;
        $data['title'] = 'Mail Verification';
        $data['body'] = 'Your OTP is: ' . $otp;

        Mail::send('mailVerification', ['data' => $data], function ($message) use ($data) {
            $message->to($data['email'])->subject($data['title']);
        });
    }
    public function requestOtp(Request $request)
    {
        $request->validate([
            'email' => 'required|email',
        ]);

        $user = User::where('email', $request->email)->first();

        if ($user) {
            $this->sendOtp($user); // Call the sendOtp method
            return response()->json(['message' => 'OTP sent successfully']);
        } else {
            return response()->json(['message' => 'User not found'], 404);
        }
    }

    public function verifyOtp(Request $request)
    {
        $request->validate([
            'email' => 'required|email',
            'otp' => 'required|digits:6',
        ]);

        $user = User::where('email', $request->email)->first();

        if (!$user) {
            return response()->json(['message' => 'User not found'], 404);
        }

        $otpData = EmailVerification::where('email', $request->email)->where('otp', $request->otp)->first();

        if ($otpData) {
            $currentTime = time();
            $time = $otpData->created_at;

            if ($currentTime >= $time && $time >= $currentTime - (90 + 5)) {
                User::where('id', $user->id)->update(['is_verified' => 1]);
                return response()->json(['message' => 'Mail has been verified']);
            } else {
                return response()->json(['message' => 'Your OTP has expired'], 422);
            }
        } else {
            return response()->json(['message' => 'You entered the wrong OTP'], 422);
        }
    }
}
