<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\ManageQuickLink;
use App\Models\User;
use App\Helpers\AdminHelper;
use Illuminate\Support\Facades\Auth;

class ManageQuickLinkController extends Controller
{

    public function index()
{
    // Get the current authenticated user
    $admin = Auth::user();
    $adminId = $admin->id;

    // Retrieve the assigned ID if available
    // Get the assigned ID using the helper function
    // Get the assigned ID using the helper function
    $assignedId = AdminHelper::getAssignedId();

    // Determine the admin ID based on the value of assignedId
    $adminId = ($assignedId === "null") ? $admin->id : $assignedId;

    // Fetch managed quick links with eager loading and sorting
    $managedQuickLinks = ManageQuickLink::where('admin_id', $adminId)
        ->with(['quickLink' => function ($query) {
            $query->orderBy('id', 'asc');
        }])
        ->orderBy('id', 'desc')
        ->get();

    // Sort managed quick links by quick link ID
    $sortedManagedQuickLinks = $managedQuickLinks->sortBy(function ($managedQuickLink) {
        return $managedQuickLink->quickLink->id;
    });

    // Map sorted quick links to link titles
    $linkTitles = $sortedManagedQuickLinks->map(function ($item) {
        return $item->quickLink->link_title;
    })->values()->all();

    // Return the result as a JSON response
    return response()->json(['managedQuickLinks' => $linkTitles]);
}

// public function index()
// {
//     $admin = Auth::user();
//     $adminId = Auth::user()->id;

//     $managedQuickLinks = ManageQuickLink::where('admin_id', $adminId)
//         ->with(['quickLink' => function ($query) {
//             $query->orderBy('id', 'asc');
//         }])
//         ->orderBy('id', 'desc')
//         ->get();

//     $assignadmins = $admin->accountadmins()->distinct()->pluck('account_ids')->toArray();

//     if ($assignadmins) {
//         foreach ($assignadmins as $assignadmin) {
//             $adminAccount = User::find($assignadmin);
//             if ($adminAccount) {
//                     $managedQuickLinks = ManageQuickLink::where('admin_id', $adminAccount->id)->with(['quickLink' => function ($query) {
//                     $query->orderBy('id', 'asc');}]) ->orderBy('id', 'desc') ->get();
//             }
//         }
//     }

//     $sortedManagedQuickLinks = $managedQuickLinks->sortBy(function ($managedQuickLink) {
//         return $managedQuickLink->quickLink->id;
//     });

//     $linkTitles = $sortedManagedQuickLinks->map(function ($item) {
//         return $item->quickLink->link_title;
//     })->values()->all();

//     return response()->json(['managedQuickLinks' => $linkTitles]);
// }



    public function store(Request $request)
    {
        // Validate request data
        $request->validate([
            'admin_id' => 'required|exists:users,id',
            'link_ids' => 'required|array',
            'link_ids.*' => 'exists:quick_links,id',
        ]);

        $adminId = $request->admin_id;

        $linkIds = $request->input('link_ids');

        foreach ($linkIds as $linkId) {
            $existingAssignment = ManageQuickLink::where('admin_id', $adminId)
                ->where('link_id', $linkId)
                ->exists();

            if (!$existingAssignment) {
                $manageQuickLink = new ManageQuickLink();
                $manageQuickLink->admin_id = $adminId;
                $manageQuickLink->link_id = $linkId;
                $manageQuickLink->save();

                $responses[] = "Link assigned successfully.";
            } else {
   
                $responses[] = "Link is already assigned to the admin.";
            }
        }

    
        return response()->json(['message' => $responses]);
    }




    public function show($adminId)
    {
        // Fetch all managed quick links
        $managedQuickLinks = ManageQuickLink::where('admin_id', $adminId)->with('quickLink:id,link_title')->get();

        // Extract only the link titles from the collection
        $linkTitles = $managedQuickLinks->map(function ($item) {
            return $item->quickLink->link_title;
        });
        // Return managed quick links as JSON response
        return response()->json(['managedQuickLinks' => $linkTitles]);
    }

    public function update(Request $request, $id)
    {
        // Validate request data
        $request->validate([
            'admin_id' => 'required|exists:users,id',
            'link_id' => 'required|exists:quick_links,id',
        ]);

        // Find and update the specified managed quick link
        $manageQuickLink = ManageQuickLink::findOrFail($id);
        $manageQuickLink->admin_id = $request->admin_id;
        $manageQuickLink->link_id = $request->link_id;
        $manageQuickLink->save();

        // Return success response
        return response()->json(['message' => 'Link updated successfully']);
    }


        public function destroy($adminId, $linkId)
    {
        try {
            // Find the specified managed quick link for the given admin and link IDs
            $manageQuickLink = ManageQuickLink::where('admin_id', $adminId)
                ->where('link_id', $linkId)
                ->firstOrFail();

            // Delete the managed quick link
            $manageQuickLink->delete();

            // Return success response
            return response()->json(['message' => 'Quick link unassigned successfully']);
        } catch (\Exception $e) {
            // Handle any exceptions, such as the managed quick link not found
            return response()->json(['message' => 'Failed to delete quick link.'], 500);
        }
    }
}
