<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\User;
use App\Models\Module;
use App\Models\NewCourse;
use Illuminate\Support\Facades\Auth;
use Illuminate\Validation\Rule;
use Illuminate\Support\Facades\Mail;
use App\Mail\CourseStartMail;
use App\Helpers\AdminHelper;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\ValidationException;


class ModuleController extends Controller
{

public function moduleCount()
{
    $admin = Auth::user();

    if ($admin->role !== 1 && $admin->role !== 2) {
        return response()->json(['message' => 'Unauthorized. Only admins can view module count.'], 403);
    }

    $assignedId = AdminHelper::getAssignedId();

    // Determine the admin ID based on the value of assignedId
    $adminId = ($assignedId === "null") ? $admin->id : $assignedId;

    $adminAccount = User::find($adminId);

    $moduleCount = $adminAccount->createdCourses()->with('modules')->get()->flatMap->modules->count();

    // $assignadmins = $admin->accountadmins()->distinct()->pluck('account_ids')->toArray();

    // if ($assignadmins) {
    //     foreach ($assignadmins as $assignadmin) {
    //         $adminAccount = User::find($assignadmin);
    //         if ($adminAccount) {
    //             $moduleCount += $adminAccount->createdCourses()->with('modules')->get()->flatMap->modules->count();
    //         }
    //     }
    // }

    return response()->json(['module_count' => $moduleCount]);
}



    public function viewModule()
    {
        $modules = Module::all();
        return response()->json(['modules' => $modules]);
    }

    //Fetch Modules for particular Course

    // public function viewModulesForCourse($courseId)
    // {
    //     $course = NewCourse::find($courseId);

    //     if (!$course) {
    //         return response()->json(['message' => 'Course not found'], 404);
    //     }

    //     $modules = $course->modules; 

    //     return response()->json(['message' => 'Modules for the course', 'modules' => $modules], 200);    
    // }


public function viewModulesForCourse(Request $request, $courseId)
{
    // Retrieve authenticated user
    $user = Auth::user();
    $admin_id = $user->admin_id;
    $admin = User::find($admin_id);
    $admin_email = $admin->email;

    if (!$user) {
        return response()->json(['error' => 'User not authenticated'], 401);
    }

    // Retrieve the course associated with the course ID
    $course = NewCourse::find($courseId);
    if (!$course) {
        return response()->json(['error' => 'Course not found'], 404);
    }

    // Check if the course is assigned to the user
    $assignedCourse = $user->courses()->where('course_id', $courseId)->first();

    if (!$assignedCourse) {
        return response()->json(['error' => 'Course not assigned to user'], 404);
    }

    // Update the task status to "In Progress" if not already in progress
    if ($assignedCourse->pivot->status !== 'In Progress') {
        $assignedCourse->pivot->status = 'In Progress';
        $assignedCourse->pivot->save();

        // Retrieve modules for the course
        $modules = $course->modules;

$defaultModules = ['No modules available'];
         Mail::to($admin_email)->send(new CourseStartMail($user,$admin,$assignedCourse));
        return response()->json([
            'message' => 'Course status updated to In Progress successfully',
            'modules' => $modules->isEmpty() ? $defaultModules : $modules
        ], 200);
    } else {
        
        $modules = $course->modules;
      $defaultModules = ['No modules available'];
        return response()->json([
            'message' => 'Course is already In Progress',
            'modules' => $modules->isEmpty() ? $defaultModules : $modules
        ], 200);
    }
}



public function viewModulesForCourseAdmin($courseId)
    {
        $course = NewCourse::find($courseId);

        if (!$course) {
            return response()->json(['message' => 'Course not found'], 404);
        }

        $modules = $course->modules;
        return response()->json(['message' => 'Modules for the course', 'modules' => $modules], 200);
    }

    public function createModule(Request $request)
    {
        $validationRules = [
            'title' => 'required|string|unique:module,title,NULL,id,course_id,' . $request->course_id,
            'course_id' => 'required|exists:new_courses,id',
            'order' => 'required|integer',
        ];

        $validationMessages = [
            'title.required' => 'The title is required.',
            'title.string' => 'The title must be a string.',
            'title.unique' => 'The title has already been taken for this course.',
            'course_id.required' => 'The course ID is required.',
            'course_id.exists' => 'The selected course is invalid.',
            'order.required' => 'The order is required.',
            'order.integer' => 'The order must be an integer.',
        ];

        try {
            // Use Validator::make() instead of $request->validate()
            $validator = Validator::make($request->all(), $validationRules, $validationMessages);

            // Manually check if validation fails
            if ($validator->fails()) {
                throw new ValidationException($validator);
            }

            // If validation passes, create the module
            $validatedData = $validator->validated();
            $module = Module::create($validatedData);

            return response()->json([
                'message' => 'Module created successfully.',
                'module' => $module,
            ], 201);

        } catch (ValidationException $e) {
            // Handle and format validation errors
            $firstErrorMessage = collect($e->errors())->flatten()->first();

            return response()->json([
                'message' => $firstErrorMessage,
                'errors' => $e->errors(),
            ], 422);

        } catch (\Throwable $e) {
            // Catch any other exceptions and log the error for debugging
            \Log::error('Module creation error: ', ['error' => $e->getMessage()]);

            return response()->json([
                'message' => 'Failed to create module.',
            ], 500);
        }



    }  

    
    // public function createModule(Request $request)
    // {
    //     $validationRules = [
    //         'title' => 'required|string|unique:module', 
    //         'course_id' => 'required|exists:new_courses,id',
    //         'order' => 'required|integer',
    //     ];

    //     $validationMessages = [
    //         'title.required' => 'The title is required.',
    //         'title.string' => 'The title must be a string.',
    //         'title.unique' => 'The title has already been taken.',
    //         'course_id.required' => 'The course ID is required.',
    //         'course_id.exists' => 'The selected course is invalid.',
    //         'order.required' => 'The order is required.',
    //         'order.integer' => 'The order must be an integer.',
    //     ];

    //     $validatedData = $request->validate($validationRules, $validationMessages);

    //     $module = Module::create($validatedData);

    //     return response()->json(['message'=>'Module Created Successfully!!','module' => $module], 201);
    // }
    // public function editModule(Request $request, $id)
    // {
    //     $data = $request->validate([
    //         'title' => 'string',
    //         'course_id' => 'exists:new_courses,id',
    //         'order' => 'integer'
    //     ]);

    //     $module = Module::find($id);

    //     if (!$module) {
    //         return response()->json(['message' => 'Module not found'], 404);
    //     }

    //     $module->update($data);

    //     return response()->json(['message' => 'Module updated successfully', 'Module' => $module], 200);
    // }

// public function editModule(Request $request, $id)
// {

//     $module = Module::find($id);

//     if (!$module) {
//         return response()->json(['message' => 'Module not found'], 404);
//     }

//     $data = $request->validate([
//         'title' => [
//             'string',
//             Rule::unique('module')->where(function ($query) use ($module) {
//             return $query->where('course_id', $module->course_id);
//         })->ignore($module->id),
//         ],
//         'course_id' => 'exists:new_courses,id',
//         'order' => 'integer'
//     ]);

//     $module->update($data);

//     return response()->json(['message' => 'Module updated successfully', 'Module' => $module], 200);
// }




public function editModule(Request $request, $id)
{
    $module = Module::find($id);

    if (!$module) {
        return response()->json(['message' => 'Module not found'], 404);
    }

    // Custom error messages
    $messages = [
        'title.string' => 'The title must be a string.',
        'title.unique' => 'The title has already been taken for this course.',
        'course_id.exists' => 'The selected course ID does not exist.',
        'order.integer' => 'The order must be an integer.',
    ];

    try {
        // Validate the request data
        $data = $request->validate([
            'title' => [
                'string',
                Rule::unique('module')->where(function ($query) use ($module) {
                    return $query->where('course_id', $module->course_id);
                })->ignore($module->id),
            ],
            'course_id' => 'exists:new_courses,id',
            'order' => 'integer',
        ], $messages); // Pass the custom messages here

        // Update the module with the validated data
        $module->update($data);

        return response()->json(['message' => 'Module updated successfully', 'Module' => $module], 200);
        
    } catch (ValidationException $e) {
        // Capture the first error message
        $firstErrorMessage = collect($e->errors())->flatten()->first();

        return response()->json([
            'message' => $firstErrorMessage,
            'errors' => $e->errors(),
        ], 422);
    } catch (\Exception $e) {
        // Handle any other exceptions
        return response()->json([
            'message' => 'An error occurred: ' . $e->getMessage(),
        ], 500);
    }
}







    public function DeleteModule($id)
    {
        $module = Module::find($id);

        if (!$module) {
            return response()->json(['message' => 'Module not found'], 404);
        }

        $module->delete();

        return response()->json(['message' => 'Module deleted successfully'], 200);
    }
}
