<?php



namespace App\Http\Controllers\api;



use App\Http\Controllers\Controller;

use Illuminate\Http\Request;

use App\Models\NewCourse;

use App\Enums\CourseStatus;

use App\Models\User;

use App\Models\Quiz;

use App\Models\NewCourseUser;

use App\Models\TaskUser;

use Illuminate\Support\Facades\Auth;

use Illuminate\Support\Facades\DB;

use Illuminate\Support\Facades\Mail;

use App\Helpers\AdminHelper;

use App\Mail\CourseReassigningMail;
use Illuminate\Support\Facades\Validator;



class NewCourseController extends Controller

{

    public function completedTasks($userId)
        {
            $user = User::find($userId);

            $completedtasks = TaskUser::where('user_id',$user->id)
            ->where('status','Completed')
            ->count();

            return response()->json(['completed_tasks' => $completedtasks]);
        }


    public function courseCount()
{
    $admin = Auth::user();

    if ($admin->role !== 1 && $admin->role !== 2) {
        return response()->json(['message' => 'Unauthorized. Only admins can view course count.'], 403);
    }

    $assignedId = AdminHelper::getAssignedId();

            // Determine the admin ID based on the value of assignedId
    $adminId = ($assignedId === "null") ? $admin->id : $assignedId; 

    $adminAccount = User::find($adminId);

    // Initialize course count with the admin's created courses count
    $courseCountTotal = $adminAccount->createdCourses()->count();

    // // $accountAdminIds = $admin->accountadmins()->pluck('account_ids')->toArray();
    // $accountAdminIds = $admin->accountadmins()->distinct()->pluck('account_ids')->toArray();

    // // Loop through each account admin ID and add their created courses count to the total
    // foreach ($accountAdminIds as $accountAdminId) {
    //     $adminAccount = User::find($accountAdminId);
    //     if ($adminAccount) {
    //         $courseCountTotal += $adminAccount->createdCourses()->count();
    //     }
    // }

    return response()->json(['course_count' => $courseCountTotal]);
}




    public function createCourse(Request $request)
    {
        $user = Auth::user();

        // Check for authorization
        if ($user->role !== 1 && $user->role !== 2) {
            return response()->json(['message' => 'Unauthorized. Only Admin or Super Admin can create courses.'], 403);
        }

        // Manually validate the request data
        $validator = Validator::make($request->all(), [
            'course_title' => 'required|string',
            'course_description' => 'required|string',
            'course_type' => 'required|string',
        ]);

        // If validation fails, return custom error message
        if ($validator->fails()) {
            $errors = $validator->errors();
            $customErrors = [];

            // Add custom message for each failed field
            if ($errors->has('course_title')) {
                $customErrors[] = 'The course title is required.';
            }
            if ($errors->has('course_description')) {
                $customErrors[] = 'The course description is required.';
            }
            if ($errors->has('course_type')) {
                $customErrors[] = 'The course type is required.';
            }

            // Return response with custom error messages
            return response()->json(['message' => implode(' ', $customErrors)], 400);
        }

        // Check if the authenticated user already has a course with the same title
        if ($user->createdCourses()->where('course_title', $request->input('course_title'))->exists()) {
            return response()->json(['message' => 'Sorry Admin, but you have already created a course with the same title.'], 400);
        }

        // Get admin data
        $admin = Auth::user();
        // $adminData = AdminHelper::getAdminAccount($admin);

        $assignedId = AdminHelper::getAssignedIdByPost($request);

        $userId = ($assignedId === "null") ? $admin->id : $assignedId;  

        // Create the course and include the user ID
        $course = $user->courses()->create([
            'course_title' => $request->input('course_title'),
            'course_description' => $request->input('course_description'),
            'course_type' => $request->input('course_type'),
            'user_id' => $userId,
        ]);

        // Return success response
        return response()->json(['message' => 'Course created successfully', 'course' => $course], 201);
    }





//this api function is for testing assign course Start
    // public function unassignedCoursestest(Request $request)
    // {
    //     $selectedUserId = $request->input('user_id');
    //     $adminUserId = Auth::id();
    //     $admin = Auth::user();
    //     $adminData = AdminHelper::getAdminAccount($admin);

    //     if($adminData){
    //        $adminUserId = $adminData ? $adminData->id : $adminUserId;
    //     }
    //     $Quiz_results= DB::table('quiz_attempts')->Where('attempt_count','3')->where('user_id',$selectedUserId)->get();
    //     print_r($Quiz_results);   die;




    // }
//this api function is for testing assign course End






    public function unassignedCourses(Request $request)
    {

        $selectedUserId = $request->input('user_id');

        // $adminUserId = Auth::id();

        $admin = Auth::user();

        // $adminData = AdminHelper::getAdminAccount($admin);
        // if($adminData){
        //     $adminUserId = $adminData ? $adminData->id : $adminUserId;
        // }

        $assignedId = AdminHelper::getAssignedId();
        
        $adminUserId = ($assignedId === "null") ? $admin->id : $assignedId;

        // print_r($selectedUserId); die;

        $quizsql = "SELECT * FROM `quiz_attempts` WHERE `user_id` = ? AND `attempt_count` = 3 ORDER BY `id` DESC";
        $quizresults = DB::select($quizsql, [$selectedUserId]);
      
        $failedCourses = [];
        $courseIds = []; 
        
        foreach ($quizresults as $qr) {
            $quizid = $qr->quiz_id;
        
            // Retrieve the quiz, lesson, module, and course
            $quiz = Quiz::find($quizid); // Using find() to avoid exception if not found
        
            if ($quiz) {
                $lesson = $quiz->lesson;
                $module = $lesson->module;
                $course = $module->course;
        
                // Check if the course is not already added
                if (!in_array($course->id, $courseIds)) {
                    $courseIds[] = $course->id;
                    $failedCourses[] = $course;
                }
            }
        }

        $sql = "
        SELECT nc.id, nc.course_title,ncu.status, nc.user_id, nc.created_at, nc.updated_at 

        FROM new_courses AS nc 

        LEFT JOIN new_course_users AS ncu ON ncu.course_id = nc.id AND ncu.user_id = ?

        WHERE nc.user_id = ? AND (ncu.status = 'Completed' OR ncu.status IS NULL)

        UNION

        SELECT nc.id, nc.course_title,ncu.status, nc.user_id, nc.created_at, nc.updated_at 

        FROM new_courses AS nc

        INNER JOIN new_course_users AS ncu ON nc.id = ncu.course_id AND ncu.user_id = ?

        WHERE nc.user_id = ? AND (ncu.status = 'Completed' OR ncu.status IS NULL)

        AND (
            ncu.status = 'Completed'

            OR (

                SELECT 1 FROM module m

                INNER JOIN module_users AS mu ON mu.module_id = m.id

                WHERE m.course_id = nc.id AND mu.`status` IS null

            )

            OR EXISTS (

                SELECT 1 FROM lessons l

                INNER JOIN module m ON l.module_id = m.id

                INNER JOIN lesson_users AS lu ON lu.lesson_id = l.id

                WHERE m.course_id = nc.id AND lu.`status` IS null

            )

            OR EXISTS (

                SELECT 1 FROM quizzes q

                INNER JOIN lessons l ON q.lesson_id = l.id

                INNER JOIN module m ON l.module_id = m.id

                WHERE m.course_id = nc.id AND q.created_at > ncu.updated_at
            )
        )
        AND nc.user_id = ?
    ";

        $combinedCourses = DB::select($sql, [
            $selectedUserId,
            $adminUserId,
            $selectedUserId,
            $adminUserId,
            $adminUserId
        ]);

        // $combinedCourses = array_merge($combinedCourses ,$failedCourses);
 

        // Ensure both $combinedCourses and $failedCourses are arrays by aman
        $combinedCourses = json_decode(json_encode($combinedCourses), true);
        $failedCourses = json_decode(json_encode($failedCourses), true);
        
        $a = array_merge($combinedCourses, $failedCourses);
        
        $data = collect($a);
        $uniqueCourses = $data->unique(function ($item) {
            return $item['id'] . $item['user_id'];
        });
        
        $all_unique_course=($uniqueCourses->values()->all());
        //End By aman
        
        return response()->json([
            'message' => 'Courses not assigned or with new content for the selected user',
            'courses' => $all_unique_course
        ], 200);

    }

    // public function courseAll(){
    //     $admin = Auth::user();
    //     $admin_id = $admin->id;

    //     $sql = "
    //     SELECT nc.id, nc.course_title, nc.status FROM new_courses AS nc LEFT JOIN new_course_users AS ncu ON ncu.course_id = nc.id WHERE ncu.user_id = ?
    //         ";


    //     $allCourses = DB::select($sql, [$admin_id]);

    //     dd($allCourses);

    //     return response()->json([

    //         'message' => 'List of all the courses',

    //         'courses' => $allCourses

    //     ], 200);

    // }


    public function courseAll()
{
    // Get the authenticated user
    $admin = Auth::user();
    $admin_id = $admin->id;

    // Define the SQL query with a parameter placeholder
    $sql = "
        SELECT nc.id, nc.course_title, nc.status 
        FROM new_courses AS nc
        LEFT JOIN new_course_users AS ncu 
        ON ncu.course_id = nc.id 
        WHERE ncu.user_id = ?
    ";

    // Execute the query with the admin_id as a parameter
    $allCourses = DB::select($sql, [$admin_id]);

    $accountadmins = $admin->accountadmins()->distinct()->pluck('account_ids')->toArray();
      if($accountadmins){
         foreach ($accountadmins as $accountadmin){
            $adminAccount = User::find($accountadmin);
            if($adminAccount){
               $allCourses += DB::select($sql, [$adminAccount->id]);
            }
         }
      }

    // Return the response with the list of courses
    return response()->json([
        'message' => 'List of all the courses',
        'courses' => $allCourses
    ], 200);
}










    public function CourseView()
    {
        $user = Auth::user();
  
        $assignedId = AdminHelper::getAssignedId();
        $admin_id = ($assignedId === "null") ? $user->id : $assignedId; 

        $courses = NewCourse::where('user_id', $admin_id)->get();
   
        // $admin = Auth::user();
        $admin=[$admin_id]; 
        $accountadmins = $admin;
        // $accountadmins = $admin->accountadmins()->distinct()->pluck('account_ids')->toArray();
        // print_r($accountadmins);  die;

        if ($accountadmins) {
            foreach ($accountadmins as $accountadmin) {
                $adminAccount = User::find($accountadmin);
                if ($adminAccount) {
                    $courses = $courses->merge(
                        NewCourse::where('user_id', $adminAccount->id)->get()
                    );
                }
            }
        }

       $formattedCourses = $courses->map(function ($course) {
        
            return [
                'id' => $course->id,
                'course_title' => $course->course_title,
                'course_description' => $course->course_description,
                'course_type' => $course->course_type,
                'status' => $course->status,
                'created_at' => $course->created_at ? $course->created_at->format('m/d/Y') : null,

            ];

        });

        return response()->json([
            'message' => 'Courses created by the authenticated user',
            'courses' => $formattedCourses

        ], 200);

    }



    public function EditCourse(Request $request, $id)
    {
        $user = Auth::user();
        if ($user->role !== 1 && $user->role !== 2) {
            return response()->json(['message' => 'Unauthorized. Only Admin or Super Admin can create courses.'], 403);
        }

        $data = $request->validate([

            'course_title' => 'string',
            'course_description' => 'string',
            'course_type' => 'string'
        ]);

        $course = NewCourse::find($id);

        if (!$course) {
            return response()->json(['message' => 'Course not found'], 404);
        }

        if ($data['course_title'] !== $course->course_title) {

            $existingCourseWithSameTitle = $user->createdCourses()->where('course_title', $data['course_title'])->where('id', '!=', $id)->exists();

            if ($existingCourseWithSameTitle) {

                return response()->json(['message' => 'Sorry Admin, but you have already created a course with the same title.'], 400);
            }
        }
        $course->update($data);
        return response()->json(['message' => 'Course updated successfully', 'course' => $course], 200);
    }


    public function DeleteCourse($id)
    {
        $course = NewCourse::find($id);

        if (!$course) {
            return response()->json(['message' => 'Course not found'], 404);
        }
        
        $course->delete();
        return response()->json(['message' => 'Course deleted successfully'], 200);
    }



    // Code for Status update



    public function updateStatus(Request $request, $userId, $courseId)

    {

        $data = $request->validate([

            'status' => 'required|in:Completed,Not Started,In Progress',

        ]);





        $user = User::find($userId);



        if (!$user) {

            return response()->json(['message' => 'User not found'], 404);

        }





        if (!$user->courses->contains($courseId)) {

            return response()->json(['message' => 'Course not assigned to the user'], 404);

        }





        $user->courses()->updateExistingPivot($courseId, ['status' => $data['status']]);



        return response()->json(['message' => 'Status updated successfully'], 200);

    }



    public function completedCourses($userId)

    {

        // Find the user by ID

        $user = User::find($userId);



        if (!$user) {

            return response()->json(['message' => 'User not found'], 404);

        }



        // Retrieve the count of completed courses for the user from the pivot table

        $completedCoursesCount = $user->courses()

            ->wherePivot('status', CourseStatus::COMPLETED) // Assuming you have a status column in your pivot table

            ->count();



        return response()->json(['completed_courses' => $completedCoursesCount], 200);

    }



    public function calculateCourseCompletionPercentage($courseId)

    {

        $user = auth()->user();



        // Find the course and its modules

        $course = NewCourse::findOrFail($courseId);

        $modules = $course->modules;



        $totalQuizzes = 0;

        $completedQuizzes = 0;



        foreach ($modules as $module) {

            // Count the quizzes in each lesson

            foreach ($module->lessons as $lesson) {

                $totalQuizzes += $lesson->quizzes()->count();

                $completedQuizzes += $user->quizzes()

                    ->where('lesson_id', $lesson->id)

                    ->wherePivot('status', 'Completed')

                    ->count();

            }

        }

        \Log::info('total quizzes ' . $totalQuizzes . ' completed quizzes ' . $completedQuizzes);

        // Calculate the overall completion percentage

        $completionPercentage = ($totalQuizzes > 0) ? ($completedQuizzes / $totalQuizzes) * 100 : 0;



        return response()->json(['completion_percentage' => $completionPercentage]);

    }



    // public function getAssignedCoursesCountAdmin()

    // {

    //     $admin = Auth::user();
    //     $accountadmins = $admin->accountadmins()->get();

    //     if (!$admin) {

    //         return response()->json(['error' => 'Admin not found'], 404);
    //     }

    //     $userIds = [$admin->id];

    //     if(isset($accountadmins)){
    //         $userIds[] = $admin->accountadmins()->distinct()->pluck('account_ids')->toArray();
    //     }



    //     $assignedTasksCount = NewCourseUser::whereIn('admin_id', $userIds)->count();

    //     return response()->json(['assignedCoursesCount' => $assignedTasksCount]);

    // }


    public function getAssignedCoursesCountAdmin()
    {
        $admin = Auth::user();

        if (!$admin) {
            return response()->json(['error' => 'Admin not found'], 404);
        }

        $assignedId = AdminHelper::getAssignedId();

        // Determine the admin ID based on the value of assignedId
        $adminId = ($assignedId === "null") ? $admin->id : $assignedId;

        // Initialize $userIds with the admin's ID
        $userIds = [$admin->id];

        // Retrieve the account admin IDs as an array
        $accountAdminIds = $admin->accountadmins()->distinct()->pluck('account_ids')->toArray();

        // Ensure $accountAdminIds is an array before merging
        if (is_array($accountAdminIds)) {
            $userIds = array_merge($userIds, $accountAdminIds);
        }

        // Get the count of assigned courses where admin_id is in $userIds
        $assignedTasksCount = NewCourseUser::where('admin_id', $adminId)->count();

        return response()->json(['assignedCoursesCount' => $assignedTasksCount]);
    }




    public function updateCourseToInProgress(Request $request, $courseId)

    {

        $employee = User::find(Auth::user()->id);



        if (!$employee) {

            return response()->json(['error' => 'Employee not found'], 404);

        }



        // Retrieve the task assigned to the employee

        $course = $employee->courses()->where('course_id', $courseId)->first();



        if (!$course) {

            return response()->json(['error' => 'Course not found'], 404);

        }



        // Check if the task status is "Not Started" before updating to "In Progress"

        if ($course->pivot->status === 'Not Started') {

            // Update the task status to "In Progress"

            $course->pivot->status = 'In Progress';

            $course->pivot->save();



            return response()->json(['message' => 'course status updated to In Progress successfully'], 200);

        } elseif ($course->pivot->status === 'In Progress') {

            return response()->json(['error' => 'course is already In Progress'], 400);

        } else {

            return response()->json(['error' => 'course cannot be updated to In Progress'], 400);

        }

    }





    public function updateCourseToCompleted(Request $request, $courseId)

    {

        $employee = User::find(Auth::user()->id);

        // dd($employee);



        if (!$employee) {

            return response()->json(['error' => 'Employee not found'], 404);

        }



        // Retrieve the course assigned to the employee

        $course = $employee->courses()->where('course_id', $courseId)->first();



        if (!$course) {

            return response()->json(['error' => 'course not found'], 404);

        }



        // Check if the course status is "In Progress" before updating to "Completed"

        if ($course->pivot->status === 'In Progress') {

            // Update the course status to "Completed"

            $course->pivot->status = 'Completed';

            $course->pivot->save();



            return response()->json(['message' => 'course status updated to Completed successfully'], 200);

        } elseif ($course->pivot->status === 'Completed') {

            return response()->json(['error' => 'course is already Completed'], 400);

        } else {

            return response()->json(['error' => 'course cannot be updated to Completed'], 400);

        }

    }



    public function deassignCourse(Request $request, $userId, $courseId)

    {

        $admin = Auth::user();

        $user = User::find($userId);

        $course = NewCourse::find($courseId);



        if (!$admin || !$user || !$course) {

            $message = [];

            if (!$admin) {

                $message[] = 'Admin not found!';

            }

            if (!$user) {

                $message[] = 'User not found!';

            }

            if (!$course) {

                $message[] = 'Course not found!';

            }

            return response()->json(['message' => $message], 404);

        }



        if ($admin->role !== 1 && $admin->role !== 2) {

            return response()->json(['message' => 'Unauthorized, only admin or super admin can de-assign tasks'], 403);

        }



        if (!$user->courses->contains($course->id)) {

            return response()->json(['message' => "Course is not assigned to the Employee"], 400);

        }



        // Detach the task from the user

        $user->courses()->detach($course->id);



        // Return a JSON response with a success message

        return response()->json(['message' => "Course de-assigned successfully"], 200);

    }



    public function getCourseCounts()

    {

        $admin = Auth::user();

        if ($admin->role !== 1 && $admin->role !== 2) {

            return response()->json(['message' => "You can not access it"], 403);

        }
        // $adminId = auth()->user()->id;
        // $adminData = AdminHelper::getAdminAccount($admin);
        //   if ($adminData) {
        //             $adminId = $adminData ? $adminData->id : $adminId;
        //     }

         $assignedId = AdminHelper::getAssignedId();

        // Determine the admin ID based on the value of assignedId
        $adminId = ($assignedId === "null") ? $admin->id : $assignedId; 

        $taskUsers = NewCourseUser::where('admin_id', $adminId)->get();





        $countInProgress = 0;

        $countCompleted = 0;

        $countNotStarted = 0;



        foreach ($taskUsers as $taskUser) {

            switch ($taskUser->status) {

                case 'In Progress':

                    $countInProgress++;

                    break;

                case 'Completed':

                    $countCompleted++;

                    break;

                case 'Not Started':

                    $countNotStarted++;

                    break;

            }

        }



        $totalTaskCount = $taskUsers->count();

        $totalEmployees = User::where('admin_id', $adminId)->count();

        $responseData = [

            'total_employees' => $totalEmployees,

            'in_progress' => $countInProgress,

            'completely_trained' => $countCompleted,

            'pending' => $countNotStarted,

        ];



        return response()->json($responseData);

    }

/** reassign courses */
// public function reassignCourse($userId, $courseId)
// {
//     $admin = Auth::user();
//     $user = User::find($userId);
//     $course = NewCourse::find($courseId);

//     if (!$admin || !$user || !$course) {
//         $message = [];
//         if (!$admin) {
//             $message[] = 'Admin not found';
//         }
//         if (!$user) {
//             $message[] = 'User not found';
//         }
//         if (!$course) {
//             $message[] = 'Course not found';
//         }

//         return response()->json(['message' => $message], 404);
//     }

//     if ($admin->role !== 1 && $admin->role !== 2) {
//         return response()->json(['message' => 'Unauthorized. Only admins can assign courses.'], 403);
//     }

//     if ($user->role === 1 || $user->role === 2) {
//         return response()->json(['message' => 'Cannot assign courses to admin users.'], 400);
//     }

//     DB::beginTransaction();
//     // dd($course);

//     try {
//         if (!$user->courses->contains($course->id)) {
//             $user->courses()->attach($course, ['admin_id' => $admin->id, 'status' => 'Not Started']);
//         }

//         foreach ($course->modules as $module) {
//             if (!$user->modules->contains($module->id)) {
//                 $user->modules()->attach($module, ['status' => 'Not Started']);
//             }

//             foreach ($module->lessons as $lesson) {
//                 if (!$user->lessons->contains($lesson->id)) {
//                     $user->lessons()->attach($lesson, ['status' => 'Not Started']);
//                 }

//                 foreach ($lesson->quizzes as $quiz) {
//                     if (!$user->quizzes->contains($quiz->id)) {
//                         $user->quizzes()->attach($quiz, ['status' => 'Not Started']);
//                     }
//                 }
//             }
//         }

//         DB::commit();

//         return response()->json(['message' => 'Course reassigned successfully'], 200);
//     } catch (\Exception $e) {
//         DB::rollback();

//         \Log::error('Failed to reassign course: ' . $e->getMessage());

//         return response()->json(['message' => 'Failed to reassign course'], 500);
//     }
// }


public function reassignCourse($userId, $courseId)
{
    $admin = Auth::user();
    $user = User::find($userId);
    $course = NewCourse::find($courseId);
    $user_email = $user->email;

    if (!$admin || !$user || !$course) {
        $message = [];
        if (!$admin) {
            $message[] = 'Admin not found';
        }
        if (!$user) {
            $message[] = 'User not found';
        }
        if (!$course) {
            $message[] = 'Course not found';
        }

        return response()->json(['message' => $message], 404);
    }

    if ($admin->role !== 1 && $admin->role !== 2) {
        return response()->json(['message' => 'Unauthorized. Only admins can assign courses.'], 403);
    }

    if ($user->role === 1 || $user->role === 2) {
        return response()->json(['message' => 'Cannot assign courses to admin users.'], 400);
    }

    DB::beginTransaction();

    try {
        // Check if the course is already assigned
        if (!$user->courses->contains($course->id)) {
            $user->courses()->attach($courseId, ['admin_id' => $admin->id, 'status' => 'Not Started']);
        } else {
            // Update the course status if already assigned
            $user->courses()->updateExistingPivot($courseId, ['status' => 'Not Started']);
        }

        // Attach or update modules, lessons, and quizzes
        foreach ($course->modules as $module) {
            if (!$user->modules->contains($module->id)) {

                $user->modules()->attach($module->id, ['status' => 'Not Started']);
            } else {
                $user->modules()->updateExistingPivot($module->id, ['status' => 'Not Started']);
            }

            foreach ($module->lessons as $lesson) {
                if (!$user->lessons->contains($lesson->id)) {
                    $user->lessons()->attach($lesson->id, ['status' => 'Not Started']);
                } else {
                    $user->lessons()->updateExistingPivot($lesson->id, ['status' => 'Not Started']);
                }

                foreach ($lesson->quizzes as $quiz) {
                    if (!$user->quizzes->contains($quiz->id)) {
                        \DB::table('quiz_attempts')
                            ->where('quiz_id', $quiz->id)
                            ->where('user_id', $user->id)
                            ->update(['attempt_count' => 0]);
                        $user->quizzes()->attach($quiz->id, ['status' => 'Not Started']);
                    } else {
                        \DB::table('quiz_attempts')
                            ->where('quiz_id', $quiz->id)
                            ->where('user_id', $user->id)
                            ->update(['attempt_count' => 0]);
                        $user->quizzes()->updateExistingPivot($quiz->id, ['status' => 'Not Started']);
                    }
                }
            }
        }

        DB::commit();
        Mail::to($user_email)->send(new CourseReassigningMail($admin,$user,$course));

        return response()->json(['message' => 'Course reassigned successfully'], 200);
    } catch (\Exception $e) {
        DB::rollback();

        \Log::error('Failed to reassign course: ' . $e->getMessage());

        return response()->json(['message' => 'Failed to reassign course'], 500);
    }
}



}

