<?php



namespace App\Http\Controllers\api;



use App\Http\Controllers\Controller;

use App\Models\Policy;

use App\Models\User;

use Illuminate\Http\Request;

use Illuminate\Support\Facades\Auth;

use Illuminate\Support\Facades\Storage;

use Carbon\Carbon;

use App\Helpers\AdminHelper;



class PolicyController extends Controller

{

    public function index()

    {
        $user = Auth::user();
        $userId = $user->id;

        if ($user->role === 2) {
        // For super admins with role 2, get all files uploaded by all admins
            $files = Policy::orderByDesc('created_at')->get();
        } elseif ($user->role === 1) {
            // For admins with role 1, get files uploaded by themselves and their associated users
            //$files = Policy::where('admin_id', $userId)->orderByDesc('created_at')->get();
            $admin = Auth::user();
            $assignedId = AdminHelper::getAssignedId();
            
            // Determine the admin ID based on the value of assignedId
            $adminId = ($assignedId === "null") ? $admin->id : $assignedId; 
            $files = Policy::where('admin_id', $adminId)->orderByDesc('created_at')->get();

            //$files = $files->merge(Policy::where('admin_id', $assignedId)->orderByDesc('created_at')->get());
        } else {
            // For non-admin users, get files uploaded by their associated admin
            $files = Policy::where('admin_id', $user->admin_id)->orderByDesc('created_at')->get();
        }

        $policies = $files->map(function ($file) {
            return [
                'id' => $file->id,
                'file_name' => $file->file_name,
                // 'filename' => substr($file->filename, 11),
                'path' => $file->path,
                'created_at' => $file->created_at,
                'admin_id' => $file->admin_id,
            ];
        });

        return response()->json(['files' => $policies]);

    }



    public function store(Request $request)

    {
        // dd($request);
        $admin = Auth::user();

        $request->validate([
            'name' => 'nullable|string|max:255',
            'description' => 'nullable|string',
            // 'files.*' => 'required|file|mimes:pdf,doc,docx,jpeg,jpg,png|max:8048',
            'files.*' => 'required|file|mimes:pdf,doc,docx,jpeg,jpg,png|max:20480',
        ]);


        $uploadedFileNames = [];
        $uploadedFilePaths = [];

        foreach ($request->file('files') as $file) {

            $filePath = $file->store('policies', 'public');

            $uploadedFileNames = $file->getClientOriginalName();
            $uploadedFilePaths = $filePath;

            $policyData = [

                'name' => $request->input('name'),

                'description' => $request->input('description'),

                'file_name' => $uploadedFileNames,

                'file_path' => $uploadedFilePaths,

                'admin_id' => $admin->id,
            ];

            $admin = Auth::user();
            //$adminData = AdminHelper::getAdminAccount($admin);
            $AdminSwitchId=$request->input('AdminSwitchId');

            $adminId = ($AdminSwitchId === "null") ? $admin->id : $AdminSwitchId; 
            if ($adminId) {
                $policyData['admin_id'] = $adminId;
            }

            // if ($adminData) {
            //     $policyData['admin_id'] = $adminData->id;
            // } 

            $policy = Policy::create($policyData);
            $uploadedFiles[] = $policy;
        }

        $responseMessage = count($uploadedFiles) === 1 ? 'Policy uploaded successfully' : 'Policies uploaded successfully';
        return response()->json(['policy' => $policy, 'message' => $responseMessage], 201);

    }





    public function show($id)
    {
        $policy = Policy::findOrFail($id);

        return response()->json($policy);

    }



    public function update(Request $request, $id)

    {

        $request->validate([

            'name' => 'nullable|string|max:255',

            'description' => 'nullable|string',

            'files.*' => 'nullable|file|mimes:pdf,doc,docx,jpeg,jpg,png|max:2048',

        ]);



        $policy = Policy::findOrFail($id);



        $uploadedFileNames = $policy->file_names ?? [];

        $uploadedFilePaths = $policy->file_paths ?? [];



        if ($request->hasFile('files')) {

            foreach ($request->file('files') as $file) {

                $filePath = $file->store('policies', 'public');

                $uploadedFileNames = $file->getClientOriginalName();

                $uploadedFilePaths = $filePath;

            }





            $policy->update([

                'name' => $request->input('name', $policy->name),

                'description' => $request->input('description', $policy->description),

                'file_name' => $uploadedFileNames,

                'file_path' => $uploadedFilePaths,

            ]);

        }



        return response()->json($policy);

    }





    public function destroy($id)

    {

        $policy = Policy::findOrFail($id);



        if ($policy->file_path) {

            Storage::delete($policy->file_path);

        }
        

        $policy->delete();



        return response()->json("Deleted Successfully", 204);

    }

}

