<?php

namespace App\Http\Controllers\Api;

use App\Models\Quiz;
use App\Models\Question;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
class QuestionController extends Controller
{


public function addquestions(Request $request, $quizId)
{

        $data = $request->validate([
            'content' => 'required|string',
            'options' => 'required|array',
            'options.*.content' => 'required|string',
            'correctOption' => 'required|integer',
        ], [
            'content.required' => 'The question content is required.',
            'options.array' => 'The options must be an array.',
            'options.*.content.required' => 'Each option must have content.',
            'correctOption.required' => 'The correct option is required.',
        ]);

        // Find the quiz
        $quiz = Quiz::findOrFail($quizId);

        // Check if the question with the same content already exists for the quiz
        $existingQuestion = $quiz->questions()->where('content', $data['content'])->first();

        if ($existingQuestion) {
            return response()->json(['message' => 'Question with the same content already exists for this quiz'], 422);
        }

        // Use a database transaction for data consistency
        DB::beginTransaction();

        // Create the question if it doesn't exist
        $question = $quiz->questions()->create(['content' => $data['content']]);

        // Create choices from the options array
        $choices = [];
        foreach ($data['options'] as $index => $option) {
            $isCorrect = $index + 1 == $data['correctOption'];
            $choices[] = ['content' => $option['content'], 'is_correct' => $isCorrect];
        }

        // Save choices to the database
        $question->choices()->createMany($choices);

        // Get correct answer
        $correctAnswer = $question->choices()->where('is_correct', true)->first();

        if ($correctAnswer) {
            DB::commit();
            return response()->json(['message' => 'Question and choices created successfully', 'data' => ['question' => $question, 'correct_answer' => $correctAnswer]], 201);
        } else {
            DB::rollBack();
            return response()->json(['message' => 'Correct answer not found for the given question'], 404);
        }
}

    public function updateQuestion(Request $request, $quizId, $questionId)
    {
        $data = $request->validate([
            'content' => 'required|string|max:255',
            'options' => 'array',
            'options.*.content' => 'required|string|max:255',
            'correctOption' => 'required|integer',
        ]);

        try {
            // Find the quiz
            $quiz = Quiz::findOrFail($quizId);

        // Check if the question with the same content already exists for the quiz
      $existingQuestion = $quiz->questions()->where('content', $data['content'])->where('id', '!=', $questionId)->first();

        if ($existingQuestion) {
            return response()->json(['message' => 'Question with the same title already exists!!'], 422);
        }
            // Find the question within the quiz
            $question = $quiz->questions()->findOrFail($questionId);

            // Use a database transaction for data consistency
            // DB::beginTransaction();

            // Update the question content
            $question->update(['content' => $data['content']]);

            // Delete existing choices
            $question->choices()->delete();

            // Create new choices from the options array
            $choices = [];
            foreach ($data['options'] as $index => $option) {
                $isCorrect = $index + 1 == $data['correctOption'];
                $choices[] = ['content' => $option['content'], 'is_correct' => $isCorrect];
            }

            // Save new choices to the database
            $question->choices()->createMany($choices);

            // Get updated correct answer
            $correctAnswer = $question->choices()->where('is_correct', true)->first();

            // Commit the transaction
            DB::commit();

            return response()->json(['message' => 'Question updated successfully', 'question' => $question, 'correct_answer' => $correctAnswer], 200);
        } catch (\Exception $e) {
            DB::rollBack();
            return response()->json(['message' => 'Failed to update question. ' . $e->getMessage()], 500);
        }
    }


public function deleteQuestion($quizId, $questionId)
{
    try {
        
        $quiz = Quiz::findOrFail($quizId);

        
        $question = $quiz->questions()->findOrFail($questionId);

        
        DB::beginTransaction();

      
        $question->choices()->delete();
        $question->delete();

      
        DB::commit();

        return response()->json(['message' => 'Question deleted successfully'], 200);
    } catch (ModelNotFoundException $e) {
        DB::rollBack();
        return response()->json(['message' => 'Question not found.'], 404);
    } catch (\Exception $e) {
        DB::rollBack();
        return response()->json(['message' => 'Failed to delete question. ' . $e->getMessage()], 500);
    }
}

}
