<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\QuickLink;
use Illuminate\Support\Facades\Auth;
class QuickLinkController extends Controller
{
    public function index()
    {
        $admin = Auth::user();
        if ($admin->role != 2) {
            return response()->json(['message' => 'only Super Admin can Access the Links']);
        }
        $quickLinks = QuickLink::all();
        return response()->json($quickLinks);
    }


    public function store(Request $request)
    {
        $request->validate([
            '*.link_title' => 'required|string',
        ]);

        $quickLinks = [];

        foreach ($request->all() as $quickLinkData) {
            $quickLinks[] = QuickLink::create($quickLinkData);
        }

        return response()->json($quickLinks, 201);
    }

    public function update(Request $request, QuickLink $quickLink)
    {
        $request->validate([
            'link_title' => 'required|string',
            'admin_id' => 'required|exists:users,id,role_id,1',
        ]);

        $quickLink->update($request->all());

        return response()->json($quickLink, 200);
    }



    public function destroy(Request $request)
    {
        $admin = Auth::user()->role;
        if ($admin != 2) {
            return response()->json(['message' => 'Unauthorized, Only Super Admin can remove the Quick Links']);
        }
        $request->validate([
            'quick_link_ids' => 'required|array',
            'quick_link_ids.*' => 'exists:quick_links,id', // Validate each ID in the array
        ]);

        foreach ($request->input('quick_link_ids') as $quickLinkId) {
            $quickLink = QuickLink::find($quickLinkId);

                $quickLink->delete();
           
        }

        return response()->json(['message' => 'links has been removed']);
    }
}
