<?php

namespace App\Http\Controllers\api;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use App\Models\User;
use Vinkla\Hashids\Facades\Hashids;
use Illuminate\Support\Facades\Mail;
use App\Mail\PendingTasksNotification;
use App\Mail\PendingCourseNotification; 
use App\Models\TaskUser;
use App\Mail\TaskAssigned;
use App\Mail\TaskStartMail;
use App\Mail\TaskStartAdmin;
use App\Mail\TaskCompleteMail;
use App\Models\Task;
use App\Helpers\AdminHelper;

class TaskController extends Controller
{

    public function taskCount()
    {

    $admin = Auth::user();
    
    if ($admin->role !== 1 && $admin->role !== 2) {
        return response()->json(['message' => 'Unauthorized. Only admins can view lesson count.'], 403);
    }

    $assignedId = AdminHelper::getAssignedId();

    // Determine the admin ID based on the value of assignedId
    $adminId = ($assignedId === "null") ? $admin->id : $assignedId;
    $adminAccount = User::find($adminId); 
    $taskCount = $adminAccount->createdTasks()->count();

        return response()->json(['task_count' => $taskCount]);
    }

    public function createTask(Request $request)
    {

    	$admin = Auth::user();
    	if($admin->role !==1 && $admin->role !==2){
    		return response()->json(['message'=>'Unauthenticated, Only admin or super admin can create tasks']);
    	}

        $data = $request->validate([
            'task_name' => 'required|string',
            'description' => 'string'
        ]);

    	// Check if admin or super admin has already created tasks with the same name 

    	if($admin->createdTasks()->where('task_name', $data['task_name'])->where('admin_id', $admin->id)->exists()){
    		return response()->json(['message'=>'Sorry admin, you have already created the task with same name please choose another name'],403);
    	}

        $admin = Auth::user();
 
        $assignedId = AdminHelper::getAssignedIdByPost($request);

        $adminId = ($assignedId === "null") ? $admin->id : $assignedId;  

        $task = Task::create([
            'task_name' => $data['task_name'], 
            'description' => $data['description'], 
            'status' => 'Not Started',
            'admin_id' => $adminId, 
        ]);

        return response()->json(['message' => 'Task created successfully', 'task' => $task], 201);
    }

    public function taskView(Request $request)
    {
    	$admin = Auth::user();
    	$tasks = Task::where('admin_id',$admin->id)->get();
        $taskCount = $tasks->count();

    	return response()->json([
    		'message'=>'task',
    		'task'=>$task,
            'taskCount'=>$taskCount
    	],200);
        // $status = $request->input('status', 'all');
        // $taskView = Task::query();

        // if ($status !== 'all') {
        //     $taskView->where('status', $status);
        // }

        // $taskView = $taskView->get();

        // return response()->json([
        //     'message' => 'Tasks filtered by status',
        //     'tasks' => $taskView
        // ], 200);
    }

    public function updateTask(Request $request, $id)
    {

        $admin = Auth::user();
        if($admin->role !==1 && $admin->role !==2){
            return response()->json(['message'=>'Unauthenticated, Only admin or super admin can create tasks']);
        }

        $data = $request->validate([
            'task_name' => 'string',
            'description' => 'string',
            
        ]);

        $task = Task::find($id);

        if (!$task) {
            return response()->json(['message' => 'task not found'], 404);
        }

        // Check if admin or super admin has already created tasks with the same name 

        if ($admin->createdTasks()
            ->where('task_name', $data['task_name'])
            ->where('id', '!=', $id)
            ->exists()) {
            
            return response()->json([
                'message' => 'Sorry Admin, you have already created a task with the same name. Please choose another name'
            ], 403);
        }

        $task->update($data);

            return response()->json(['message' => 'task updated successfully', 'task' => $task], 200);
        
    }

    public function DeleteTask($id)
    {
        $task = Task::find($id);

        if (!$task) {
            return response()->json(['message' => 'Task not found'], 404);
        }

        $task->delete();

        return response()->json(['message' => 'Task deleted successfully'], 200);
    }

    // old code

    // public function assignTasks(Request $request, $userId)
// {
//     $admin = Auth::user();
//     $user = User::find($userId);

//     if (!$admin || !$user) {
//         $message = [];
//         if (!$admin) {
//             $message[] = 'Admin not found!';
//         }
//         if (!$user) {
//             $message[] = 'User not found!';
//         }
//         return response()->json(['message' => $message], 404);
//     }

//     if ($admin->role !== 1 && $admin->role !== 2) {
//         return response()->json(['message' => 'Unauthorized, only admin or super admin can assign the task'], 403);
//     }
//     if ($user->role === 1 || $user->role === 2) {
//         return response()->json(['message' => 'Cannot assign tasks to admins or super admins']);
//     }
//     $taskIds = $request->input('taskIds');

//     if (!is_array($taskIds)) {
//         return response()->json(['message' => 'Task IDs must be provided as an array'], 400);
//     }
//     foreach ($taskIds as $taskId) {
//         $task = Task::find($taskId);

//         if (!$task) {
//             return response()->json(['message' => 'Task not found'], 404);
//         }

//         if ($user->tasks->contains($task->id)) {
//             return response()->json(['message' => "Task already assigned to the Employee"], 400);
//         }

//         $user->tasks()->attach($task, ['admin_id' => $admin->id, 'status' => 'Not Started']);
//     }
//     try {
//         Mail::to($user->email)->send(new TaskAssigned($user, $task));
//     } catch (\Exception $e) {
//         \Log::error('Error sending email: ' . $e->getMessage());
//         return response()->json(['message' => 'Failed to send email', 'error' => $e->getMessage()], 500);
//     }

//     return response()->json(['message' => 'Tasks assigned successfully'], 200);
// }


public function assignTasks(Request $request, $userId)
{
    $admin = Auth::user();
    $user = User::find($userId);
    $user_email = $user->email;

    if (!$admin || !$user) {
        $message = [];
        if (!$admin) {
            $message[] = 'Admin not found!';
        }
        if (!$user) {
            $message[] = 'User not found!';
        }
        return response()->json(['message' => $message], 404);
    }

    if ($admin->role !== 1 && $admin->role !== 2) {
        return response()->json(['message' => 'Unauthorized, only admin or super admin can assign the task'], 403);
    }
    if ($user->role === 1 || $user->role === 2) {
        return response()->json(['message' => 'Cannot assign tasks to admins or super admins']);
    }

    // Retrieve task IDs from the request payload
    $taskIds = $request->input('taskIds');
    // $tasksDetails = Task::find($taskIds);

    // Iterate through the array of task IDs
    foreach ($taskIds as $taskId) {
        $task = Task::find($taskId);

        if (!$task) {
            return response()->json(['message' => 'Task not found'], 404);
        }

        // Check if the task is already assigned to the user
        if ($user->tasks->contains($task->id)) {
            return response()->json(['message' => "Task already assigned to the Employee"], 400);
        }

        // Attach the task to the user with the admin ID and the default status 'Not Started'
        $user->tasks()->attach($task, ['admin_id' => $admin->id, 'status' => 'Not Started']);
        Mail::to($user_email)->send(new TaskStartMail($admin,$user,$task));
    }

    $taskIds = $request->input('taskIds');

    if(count($taskIds)==1){
        $message = "Task assigned successfully";
    }else{
        $message = "Tasks assigned successfully";
    }

    // Return a JSON response with a success message
    
    return response()->json(['message' => $message], 200);
}
/** Code By San | send mail over asisgn task */
                  



public function deassignTask(Request $request, $userId, $taskId)
{
    $admin = Auth::user();
    $user = User::find($userId);
    $task = Task::find($taskId);

    if (!$admin || !$user || !$task) {
        $message = [];
        if (!$admin) {
            $message[] = 'Admin not found!';
        }
        if (!$user) {
            $message[] = 'User not found!';
        }
        if (!$task) {
            $message[] = 'Task not found!';
        }
        return response()->json(['message' => $message], 404);
    }

    if ($admin->role !== 1 && $admin->role !== 2) {
        return response()->json(['message' => 'Unauthorized, only admin or super admin can de-assign tasks'], 403);
    }

    if (!$user->tasks->contains($task->id)) {
        return response()->json(['message' => "Task is not assigned to the Employee"], 400);
    }

    // Detach the task from the user
    $user->tasks()->detach($task->id);

    // Return a JSON response with a success message
    return response()->json(['message' => "Task de-assigned successfully"], 200);
}


    public function tasksForEmployee(){
    	$employee = User::find(Auth::user()->id);

   if (!$employee) {
            
            return response()->json(['error' => 'Employee not found'], 404);
        }

        $assignedTasks = $employee->tasks;
        $assignedTasksCount = $assignedTasks->count();
        return response()->json(['totalTasks'=>$assignedTasksCount,'message'=>'Assigned tasks', 'assignedTasks'=>$assignedTasks],200);

    }


public function updateTaskStatus(Request $request, $taskId) {
    $employee = User::find(Auth::user()->id);

    if (!$employee) {
        return response()->json(['error' => 'Employee not found'], 404);
    }

    // Retrieve the task assigned to the employee
    $task = $employee->tasks()->where('task_id', $taskId)->first();

    if (!$task) {
        return response()->json(['error' => 'Task not found'], 404);
    }

    // Update the task status to "In Progress"
    $task->pivot->status = 'In Progress';
    $task->pivot->save();

    return response()->json(['message' => 'Task status updated successfully'], 200);
}

public function updateTaskToInProgress(Request $request, $taskId) { 
    $employee = User::find(Auth::user()->id);
    $adminId = $employee->admin_id;
    $admin = User::where('id', $adminId)->first();
    $admin_email = $admin->email;

    if (!$employee) {
        return response()->json(['error' => 'Employee not found'], 404);
    }

    // Retrieve the task assigned to the employee
    $task = $employee->tasks()->where('task_id', $taskId)->first();

    if (!$task) {
        return response()->json(['error' => 'Task not found'], 404);
    }

    // Check if the task status is "Not Started" before updating to "In Progress"
    if ($task->pivot->status === 'Not Started') {
        // Update the task status to "In Progress"
        $task->pivot->status = 'In Progress';
        $task->pivot->save();

        Mail::to($admin_email)->send(new TaskStartAdmin($employee,$admin,$task));
        return response()->json(['message' => 'Task status updated to In Progress successfully'], 200);
    } elseif ($task->pivot->status === 'In Progress') {
        return response()->json(['error' => 'Task is already In Progress'], 400);
    } else {
        return response()->json(['error' => 'Task cannot be updated to In Progress'], 400);
    }
}

public function updateTaskToCompleted(Request $request, $taskId) {
    $employee = User::find(Auth::user()->id);
    $user = Auth::user();
    $admin = User::find($employee);
    $admin_id = $employee->admin_id;
    $admin_email = User::where('id', $admin_id)->value('email');

    if (!$employee) {
        return response()->json(['error' => 'Employee not found'], 404);
    }

    // Retrieve the task assigned to the employee
    $task = $employee->tasks()->where('task_id', $taskId)->first();

    if (!$task) {
        return response()->json(['error' => 'Task not found'], 404);
    }

    // Check if the task status is "In Progress" before updating to "Completed"
    if ($task->pivot->status === 'In Progress') {
        // Update the task status to "Completed"
        $task->pivot->status = 'Completed';
        $task->pivot->save();

          Mail::to($admin_email)->send(new TaskCompleteMail($admin,$user,$task));

        return response()->json(['message' => 'Task status updated to Completed successfully'], 200);
    } elseif ($task->pivot->status === 'Completed') {
        return response()->json(['error' => 'Task is already Completed'], 400);
    } else {
        return response()->json(['error' => 'Task cannot be updated to Completed'], 400);
    }
}

    public function sendNotification($userId)
    {
        // Retrieve user based on $userId
        $user = User::find($userId);

        if (!$user) {
            return response()->json(['error' => 'User not found'], 404);
        }

        // Send email notification to the user
        try {
            Mail::to($user->email)->send(new PendingTasksNotification($user));
            return response()->json(['message' => 'Notification sent successfully'], 200);
        } catch (\Exception $e) {
            return response()->json(['error' => 'Failed to send notification', 'message' => $e->getMessage()], 500);
        }
    }

    public function sendCourseNotification($userId){
        $user = User::find($userId);
        if(!$user){
            return response()->json(['error'=>'User not found']);
        }
        try {
            Mail::to($user->email)->send(new PendingCourseNotification($user));
            return response()->json(['message' => 'Notification sent successfully'], 200);
        } catch (\Exception $e) {
            return response()->json(['error' => 'Failed to send notification', 'message' => $e->getMessage()], 500);
        }
    }

 public function getUserTasks($userId)
    {
        $user = User::findOrFail($userId);

        $tasks = $user->tasks()->withPivot('status')->get();

        $countInProgress = 0;
        $countCompleted = 0;
        $countNotStarted = 0;


        foreach ($tasks as $task) {
            switch ($task->pivot->status) {
                case 'In Progress':
                    $countInProgress++;
                    break;
                case 'Completed':
                    $countCompleted++;
                    break;
                case 'Not Started':
                    $countNotStarted++;
                    break;
            }
        }

        $totalTaskCount = $tasks->count();

       
        $responseData = [
            'total_task_count' => $totalTaskCount,
            'count_in_progress' => $countInProgress,
            'count_completed' => $countCompleted,
            'count_not_started' => $countNotStarted,
          
        ];

        return response()->json($responseData);
    }


public function getTaskCount()
    {
      $admin = Auth::user();
      if($admin->role !== 1 && $admin->role !==2){
      	return response()->json(['message'=>"You can not access it"],403);
      }
      $taskUsers = TaskUser::where('admin_id', $admin->id)->get();


        $countInProgress = 0;
        $countCompleted = 0;
        $countNotStarted = 0;

        foreach ($taskUsers as $taskUser) {
            switch ($taskUser->status) {
                case 'In Progress':
                    $countInProgress++;
                    break;
                case 'Completed':
                    $countCompleted++;
                    break;
                case 'Not Started':
                    $countNotStarted++;
                    break;
            }
        }

        $totalTaskCount = $taskUsers->count();

        $responseData = [
            'total_task' => $totalTaskCount,
            'in_progress' => $countInProgress,
            'completed' => $countCompleted,
            'not_started' => $countNotStarted,
        ];

        return response()->json($responseData);
    }

    public function getTaskCounts()
    {
      $admin = Auth::user();
      if($admin->role !== 1 && $admin->role !==2){
      	return response()->json(['message'=>"You can not access it"],403);
      }
      // $adminId = auth()->user()->id;
      // $adminData = AdminHelper::getAdminAccount($admin);
      // if ($adminData) {
      //           $adminId = $adminData ? $adminData->id : $adminId;
      //   }

       $assignedId = AdminHelper::getAssignedId();

        // Determine the admin ID based on the value of assignedId
        $adminId = ($assignedId === "null") ? $admin->id : $assignedId; 
        $taskUsers = TaskUser::where('admin_id', $adminId )->get();


        $countInProgress = 0;
        $countCompleted = 0;
        $countNotStarted = 0;

        foreach ($taskUsers as $taskUser) {
            switch ($taskUser->status) {
                case 'In Progress':
                    $countInProgress++;
                    break;
                case 'Completed':
                    $countCompleted++;
                    break;
                case 'Not Started':
                    $countNotStarted++;
                    break;
            }
        }

        $totalTaskCount = $taskUsers->count();

        $responseData = [
            'total_task' => $totalTaskCount,
            'in_progress' => $countInProgress,
            'completed' => $countCompleted,
            'not_started' => $countNotStarted,
        ];

        return response()->json($responseData);
    }



    public function getTaskCountsUser()
    {
      $user = Auth::user();

      $taskUsers = TaskUser::where('user_id', $user->id)->get();


        $countInProgress = 0;
        $countCompleted = 0;
        $countNotStarted = 0;

        foreach ($taskUsers as $taskUser) {
            switch ($taskUser->status) {
                case 'In Progress':
                    $countInProgress++;
                    break;
                case 'Completed':
                    $countCompleted++;
                    break;
                case 'Not Started':
                    $countNotStarted++;
                    break;
            }
        }

        $totalTaskCount = $taskUsers->count();

        $responseData = [
            'total_task' => $totalTaskCount,
            'in_progress' => $countInProgress,
            'completed' => $countCompleted,
            'not_started' => $countNotStarted,
        ];

        return response()->json($responseData);
    }

public function completedTasks($userId)
    {
 
        $user = User::find($userId);

        if (!$user) {
            return response()->json(['message' => 'User not found'], 404);
        }

        $completedTasksCount = $user->tasks()
            ->wherePivot('status', 'completed') 
            ->count();

        return response()->json(['completed_tasks' => $completedTasksCount], 200);
    }

        public function getAssignedTasksCount($userId)
    {
        
        $user = User::find($userId);

        
        if (!$user) {
          
            return response()->json(['error' => 'User not found'], 404);
        }

     
        $assignedTasksCount = $user->tasks()->count();

        
        return response()->json(['assignedTasksCount' => $assignedTasksCount]);
    }

    public function getAssignedTasksCountAdmin()
    {
        
        $admin = Auth::user();

        if (!$admin) {
          
            return response()->json(['error' => 'Admin not found'], 404);
        }

         $assignedId = AdminHelper::getAssignedId();

        // Determine the admin ID based on the value of assignedId
        $adminId = ($assignedId === "null") ? $admin->id : $assignedId; 

        $assignedTasksCount = TaskUser::where('admin_id', $adminId)->count();

        return response()->json(['assignedTasksCount' => $assignedTasksCount]);
    }

    public function unassignedTasks(Request $request)
    {
        $selectedUserId = $request->input('user_id');
        $admin=Auth::user();
        
        $assignedId = AdminHelper::getAssignedIdByPost($request);

        $adminId = ($assignedId === "null") ? $admin->id : $assignedId; 

        // print_r($adminId); die;

            $newTasks = Task::whereNotIn('id', function ($query) use ($selectedUserId) {
            $query->select('task_id')
                ->from('task_users')
                ->where('user_id', $selectedUserId);
            })
            ->where('admin_id', $adminId)
            ->get();

        // print_r($newTasks); die;
        // $admin = Auth::user();
        // $accountadmins = $admin->accountadmins()->distinct()->pluck('account_ids')->toArray();
        // $newTasks = collect(); // Initialize an empty collection
        $accountadmins = [$adminId];

        if ($accountadmins) {
            foreach ($accountadmins as $accountadmin) {
                $adminAccount = User::find($accountadmin);
                if ($adminAccount) {
                    $tasks = Task::whereNotIn('id', function ($query) use ($selectedUserId) {
                        $query->select('task_id')
                            ->from('task_users')
                            ->where('user_id', $selectedUserId);
                    })
                    ->where('admin_id', $adminAccount->id)
                    ->get();
                    
                    $newTasks = $newTasks->merge($tasks);
                }
            }
        }


        return response()->json([
            'message' => 'Tasks not assigned to the selected user',
            'tasks' => $newTasks
        ], 200);
    }

}
