<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\TemplateCategory;
use Illuminate\Support\Facades\Auth;
use Illuminate\Validation\Rule;
use App\Helpers\AdminHelper;
use App\Models\User;
class TemplateCategoryController extends Controller
{

    public function index()
    {

        $adminId = auth()->user()->id;
        $admin = Auth::user();
        // $adminData = AdminHelper::getAdminAccount($admin);

        // if ($adminData) {
        //         $adminId = $adminData ? $adminData->id : $adminId;
        // }

        $assignedId = AdminHelper::getAssignedId();

        $adminId = ($assignedId === "null") ? $admin->id : $assignedId;

        // Get all template categories with the count of associated templates
        $categories = TemplateCategory::where('admin_id', $adminId)
        ->withCount('templates')->get();

        // Format the response
        $formattedCategories = $categories->map(function ($category) {
            return [
                'id' => $category->id,
                'name' => $category->name,
                'count' => $category->templates_count,
            ];
        });

        return response()->json(['categories' => $formattedCategories]);
    }



    public function store(Request $request)
    {
        $admin = Auth::user();

        $request->validate([
            'name' => 'required|string|max:255',
        ]);

        $categoryData = $request->only('name');
        // $categoryData['admin_id'] = $admin->id;
        // $adminData = AdminHelper::getAdminAccount($admin);

        // if ($adminData) {
        //         $categoryData['admin_id'] = $adminData ? $adminData->id : $admin->id;
        //         $admin->id = $categoryData['admin_id'];
        // }

        $assignedId = AdminHelper::getAssignedIdByPost($request);

        // Determine the admin ID based on the value of assignedId
        $adminId = ($assignedId === "null") ? $admin->id : $assignedId; 

        $categoryData['admin_id'] = $adminId;


        // Check if admin or super admin has already created tasks with the same name 

        if($admin->createdTemplateCategory()->where('name', $categoryData['name'])->where('admin_id', $admin->id)->exists()){
            return response()->json(['message'=>'Sorry admin, you have already created the template category with same name please choose another name'],403);
        }
        $category = TemplateCategory::create($categoryData);

        return response()->json(['message'=>'Template Category added successfully!!','category' => $category], 201);
    }


public function update(Request $request, $categoryId)
{
    $admin = Auth::user();
    $request->validate([
        'name' => 'required|string|max:255',
    ]);

    $category = TemplateCategory::find($categoryId);
    
    if(!$category) {
        return response()->json(['message'=>'Category Not Found!!']);
    }

    // Check if the new name conflicts with existing names
    // $existingCategory = TemplateCategory::where('name', $request->name)
    //                                     ->where('id', '!=', $categoryId)
    //                                     ->first();

    $existingCategory = TemplateCategory::where('name', $request->name)
                                        ->where('admin_id', '=', $admin->id)
                                        ->where('id', '!=', $categoryId)
                                        ->first();

    if($existingCategory) {
        return response()->json(['message'=>'Category name already exists!'], 400);
    }

    // Update the category
    $category->update($request->only('name'));

    return response()->json(['message'=>'Template Category updated successfully!!','category' => $category], 200);
}


public function destroy($categoryId)
{
 
    // Attempt to delete the category

    $category = TemplateCategory::find($categoryId);

    $category->delete();

    return response()->json(['message' => 'Template Category deleted successfully!!'], 200);
}

}
