<?php

namespace App\Http\Controllers\Api;

use Carbon\Carbon;
use App\Models\EmailVerification;
use Illuminate\Support\Facades\Mail;
use App\Http\Controllers\Controller;
use App\Models\User;
use Illuminate\Http\Request;



class TwoFAController extends Controller
{
    public function sendOtp(User $user)
    {
        $otp = rand(100000, 999999);

        EmailVerification::updateOrCreate(
            ['email' => $user->email],
            [
                'email' => $user->email,
                'otp' => $otp,
                'created_at' => Carbon::now(),
            ]
        );

        $data['email'] = $user->email;
        $data['title'] = 'Mail Verification';
        $data['body'] = 'Your OTP is: ' . $otp;

        try {
            Mail::send('mailVerification', ['data' => $data], function ($message) use ($data) {
                $message->to($data['email'])->subject($data['title']);
            });
            // Log success
            \Log::info('OTP sent successfully for user: ' . $user->email);
            return response()->json(['message' => 'OTP sent successfully']);
        } catch (\Exception $e) {
            // Log error
            \Log::error('Failed to send OTP for user: ' . $user->email . '. Error: ' . $e->getMessage());
            return response()->json(['message' => 'Failed to send OTP. Please try again.'], 500);
        }
    }

    public function requestOtp(Request $request)
    {
        $request->validate([
            'email' => 'required|email',
        ]);

        $user = User::where('email', $request->email)->first();

        if ($user) {
            $this->sendOtp($user);

            return response()->json(['message' => 'OTP sent successfully']);
        } else {
            return response()->json(['message' => 'User not found'], 404);
        }
    }

    public function verifyOtp(Request $request, User $user)
    {
        $request->validate([
            'email' => 'required|email',
            'otp' => 'required|digits:6',
        ]);

        if (!$user) {
            return response()->json(['message' => 'User not found'], 422);
        }

        $otpData = EmailVerification::where('email', $request->email)->where('otp', $request->otp)->first();

        if ($otpData) {
            $currentTime = Carbon::now();
            $time = $otpData->created_at;

            if ($currentTime >= $time && $time >= $currentTime->subSeconds(90)) {
                User::where('id', $user->id)->update(['is_verified' => 1]);

                return response()->json(['message' => 'Mail has been verified']);
            } else {
                return response()->json(['message' => 'Your OTP has expired'], 422);
            }
        } else {
            return response()->json(['message' => 'You entered the wrong OTP'], 422);
        }
    }
}
