<?php

namespace App\Services;

use App\Models\EmailTemplate;
use Illuminate\Support\Facades\Mail;
use App\Models\EmailTracking;

class EmailService
{
    public function sendEmail($templateId, $to, $data = [])
    {
        
        $template = EmailTemplate::findOrFail($templateId);
        \Log::info('content before replacement ' . $template->content);
        $content = $this->replaceVariables($template->content, $data);
        \Log::info('content after replacement ' . $content);
        \Log::info('template ' . $template);
        \Log::info('content ' . $content);

        // Generate unique tracking identifiers
        $openTrackingIdentifier = uniqid();
        $clickTrackingIdentifier = uniqid();
        \Log::info('open track identifier ' . $openTrackingIdentifier);
        \Log::info('click track identifier ' . $clickTrackingIdentifier);

        // Replace tracking placeholders with actual tracking URLs
        $openTrackingUrl = route('email.open', ['id' => $templateId, 'uid' => $openTrackingIdentifier, 'email' => $to]);
        
        $clickTrackingUrl = route('email.click', ['id' => $templateId, 'uid' => $clickTrackingIdentifier, 'email' => $to]);
        \Log::info('open track url ' . $openTrackingUrl);
        \Log::info('click track url ' . $clickTrackingUrl);

        // Embed open tracking pixel
        $openTrackingPixel = '<img src="' . $openTrackingUrl . '" alt="Pixel">';
        \Log::info('open track pixel ' . $openTrackingPixel);
        $content = str_replace("{{open_tracking_pixel}}", $openTrackingPixel, $content);
        \Log::info('open content ' . $content);

        // Replace click tracking link
        $content = str_replace("{{click_tracking_link}}", $clickTrackingUrl, $content);
        \Log::info('click content ' . $content);

        Mail::send([], [], function ($message) use ($to, $template, $content, $data) {
            $message->to($to)
                ->subject($data['subject'] ?? $template->subject)
                ->setBody($content, 'text/html');

            // Set sender name and address
            $senderName = $data['sender_name'] ?? $template->sender_name;
            $senderAddress = $data['sender_address'] ?? $template->sender_address;

            if ($senderName && $senderAddress) {
                $message->from($senderAddress, $senderName);
            }
        });

        $this->saveTrackingData($templateId, $to, $openTrackingIdentifier, $clickTrackingIdentifier);
    }

    private function replaceVariables($content, $data)
    {
        foreach ($data as $key => $value) {
            $content = str_replace("{{ $key }}", $value, $content);
        }

        return $content;
    }

    private function saveTrackingData($templateId, $recipient, $openTrackingIdentifier, $clickTrackingIdentifier)
    {
        // Save tracking data to the EmailTracking table or your preferred storage mechanism
        EmailTracking::create([
            'template_id' => $templateId,
            'recipient' => $recipient,
            'email' => $recipient,
            'open_tracking_identifier' => $openTrackingIdentifier,
            'click_tracking_identifier' => $clickTrackingIdentifier,
        ]);
    }
}