<?php

namespace App\Services;

use App\Models\EmailTemplate;
use Illuminate\Support\Facades\Mail;
use App\Models\EmailTracking;
use App\Models\EmailSentTrack;

class EmailService
{
    public function sendEmail($templateId, $to, $adminId, $data = [])
    {
        $userId = $data['user_id'];
        
        $template = EmailTemplate::findOrFail($templateId);
        $pageId = $template->landing_page_id;
        \Log::info('content before replacement ' . $template->content);
        $content = $this->replaceVariables($template->content, $data);
        $pattern = '/<a\s+(?:[^>]*?\s+)?href=(["\'])(.*?)\1/';
        $content = preg_replace_callback($pattern, function ($matches) use ($templateId, $to,$userId,$pageId) {
            $newHref = 'https://learning.projects-codingbrains.com/click-rate?temp_id='.$templateId.'&email='.$to. '&user_id='.$userId. '&page_id='.$pageId. '&url=' . $matches[2];

            $target =  'target="_blank"';

            return "<a href='$newHref' $target";
        }, $content);


        // Replace tracking placeholders with actual tracking URLs
        $openTrackingUrl = route('email.open', ['templateId' => $templateId, 'email' => $to,  'userId' => $data['user_id'] , 'url'=>'www.google.com']);


        // Embed open tracking pixel
        $openTrackingPixel = '<img src="' . $openTrackingUrl . '" alt="" style="display:none; width:1px; height:1px;">';

        \Log::info('image details' . $openTrackingPixel);
        $content = str_replace("{{open_tracking_pixel}}", $openTrackingPixel, $content);
        \Log::info('Final Email Content', ['content' => $content]);


        Mail::send([], [], function ($message) use ($to, $template, $content, $data) {
            $message->to($to)
                ->subject($data['subject'] ?? $template->subject)
                ->setBody($content, 'text/html');

            // Set sender name and address
            $senderName = $data['sender_name'] ?? $template->sender_name;
            $senderAddress = $data['sender_address'] ?? $template->sender_address;

            if ($senderName && $senderAddress) {
                $message->from($senderAddress, $senderName);
            }
        });

        $this->saveTrackingData($templateId, $to, $adminId);
    }

    private function replaceVariables($content, $data)
    {
        foreach ($data as $key => $value) {
            $content = str_replace("{{ $key }}", $value, $content);
        }

        return $content;
    }

    private function saveTrackingData($templateId, $recipient, $adminId)
    {
       
        EmailSentTrack::create([
            'template_id' => $templateId,
            'admin_id' => $adminId,
            'recepient' => $recipient,
        ]);
    }
}