<?php

use Illuminate\Http\Request;
use App\Http\Controllers\Api\AuthController;
use App\Http\Controllers\Api\UserController;
use Illuminate\Support\Facades\Route;
use App\Http\Controllers\Api\TaskController;
use App\Http\Controllers\Api\NewCourseController;
use App\Http\Controllers\Api\LessonController;
use App\Http\Controllers\Api\ModuleController;
use App\Http\Controllers\Api\FileUploadController;
use App\Http\Controllers\Api\PhishingSimulationController;
use App\Http\Controllers\Api\TwoFAController;
use App\Http\Controllers\dummycontroller;
use App\Http\Controllers\Api\ForgotPasswordController;
use App\Http\Controllers\Api\QuizController;
use App\Http\Controllers\Api\QuestionController;
use App\Http\Controllers\Api\ChoiceController;
use App\Http\Controllers\Api\EmailTemplateController;
use App\Http\Controllers\Api\EmailTrackingController;
use App\Http\Controllers\Api\TemplateCategoryController;
use App\Http\Controllers\Api\LandingPageController;
use App\Http\Controllers\Api\LandingPageCategoryController;
use App\Http\Controllers\Api\QuickLinkController;
use App\Http\Controllers\Api\ManageQuickLinkController;
use App\Http\Controllers\Api\EmailSentTrackController;
use Illuminate\Support\Facades\Artisan;
use App\Http\Controllers\Api\PolicyController;
use App\Http\Controllers\Api\AccountAdminController;
/*

|--------------------------------------------------------------------------

| API Routes

|--------------------------------------------------------------------------

|

| Here is where you can register API routes for your application. These

| routes are loaded by the RouteServiceProvider within a group which

| is assigned the "api" middleware group. Enjoy building your API!

|

*/

// Optimization through Routes - 

 Route::get('optimize', function () {
        Artisan::call('optimize:clear');
        Artisan::call('route:clear');
        Artisan::call('cache:clear ');
        Artisan::call('config:clear');
        return "Optimization cleared!";
    });

//Running migrations through Routes

Route::get('/run-migrations', function () {
    Artisan::call('migrate:run');
    return 'Migrations successfully run.';
});


Route::middleware('auth:sanctum')->group(function () {

    Route::post('/logout', [AuthController::class, 'logout']);

    Route::get('/user', function (Request $request) {

        return $request->user();

    });



// Multiple admins API

Route::get('/show-admin-list', [UserController::class, 'showAdminList']);
Route::get('/get-current-userid/{id}', [UserController::class, 'getCurrentUser']);
Route::get('/test', [UserController::class, 'test']);


// Logo Related Routes - 

Route::post('/upload-logo', [UserController::class, 'uploadLogo'])->name('upload.logo');
Route::get('/get-logo', [UserController::class, 'getLogo'])->name('get.logo');

// Routes for super admin to manage admins- 

Route::post('/create-admin', [AuthController::class, 'createAdmin']);
Route::get('/show-admin', [UserController::class, 'showAdmin']);
Route::put('/admin/update/{id}', [UserController::class, 'updateAdmin']);
Route::get('/show-admin/{id}', [UserController::class, 'editAdmin']);
Route::delete('/admin/delete/{id}', [UserController::class, 'deleteAdmin']);
Route::get('/admincount', [UserController::class, 'adminCount']);
/** San Codes */
Route::get('/employee-progress/{id}', [UserController::class, 'getEmployeeProgress']);

// 2FA Functionality - 
Route::post('/enable-two-fa', [AuthController::class, 'enableTwoFa']);
Route::post('/disable-two-fa', [AuthController::class, 'disableTwoFa']);

Route::post('changepassword', [AuthController::class, 'changePassword']);
Route::get('/get-course-status', [NewCourseController::class, 'getCourseCounts']);

Route::post('/reassign-courses/{cid}/{uid}', [NewCourseController::class, 'reassignCourse']);

// Route::get('/users', UserController::class);
Route::apiResource('/users', UserController::class);

// Profile pic related Routes - 
Route::get('/users-by-name-for-admin', [UserController::class,'FilterByNameForAdmin']);
Route::get('/users-by-name', [UserController::class,'FilterByName']);
Route::post('profilepic', [UserController::class, 'updateProfilePic']);
Route::get('getProfilePic', [UserController::class, 'getProfilePic']);
Route::post('/questions/{questionId}/submit-response', [ChoiceController::class, 'submitUserResponse']);
Route::post('/submit-responses/{quizId}', [ChoiceController::class, 'submitUserResponses']);


// Routes for Course - 

Route::post('createcourse', [NewCourseController::class, 'CreateCourse']);
Route::put('/editcourse/{id}', [NewCourseController::class, 'EditCourse']);
Route::delete('deletecourse/{id}', [NewCourseController::class, 'DeleteCourse']);
Route::get('courseview', [NewCourseController::class, 'CourseView']);
Route::post('/assign-course/{userId}/{courseId}', [UserController::class, 'assignCourse']);
Route::get('course-not-assigned-view', [NewCourseController::class, 'unassignedCourses']);

Route::get('course-all', [NewCourseController::class, 'courseAll']);

Route::put('/courses/{courseId}/update-to-in-progress', [NewCourseController::class,'updateCourseToInProgress']);
Route::put('/courses/{courseId}/update-to-completed', [NewCourseController::class,'updateCourseToCompleted']);

Route::get('/courses/{courseId}/completion-percentage', [NewCourseController::class, 'calculateCourseCompletionPercentage']);
Route::get('/quizzes/{quizId}/questions-with-choices', [QuizController::class, 'getQuestionsWithChoices'])->middleware('check.quiz.completion');



Route::get('/user/{userId}/assigned-courses', [UserController::class, 'getAssignCourses']);
Route::get('/getAssignedCoursesCount/{id}', [UserController::class, 'getAssignedCoursesCount']);
Route::get('/getAssignedTasksCount/{id}', [TaskController::class, 'getAssignedTasksCount']);
Route::put('/users/{userId}/courses/{courseId}/update-status', [NewCourseController::class, 'updateStatus']);
Route::delete('/users/{userId}/deassign-course/{courseId}', [NewCourseController::class, 'deassignCourse']);
// Routes for creating modules 

Route::post('createmodule', [ModuleController::class, 'createModule']);
Route::get('viewmodule', [ModuleController::class, 'viewModule']);
Route::put('editmodule/{id}', [ModuleController::class, 'editModule']);
Route::delete('deletemodule/{id}', [ModuleController::class, 'deleteModule']);
Route::get('/courses-admin/{courseId}/modules', [ModuleController::class, 'viewModulesForCourseAdmin']);
Route::get('/courses/{courseId}/modules', [ModuleController::class, 'viewModulesForCourse'])->middleware('check.course.completion');


// Routes for lessions 

Route::post('createlesson', [LessonController::class, 'createLesson']);
Route::get('viewlessons', [LessonController::class, 'viewLessons']);
Route::put('editlesson/{id}', [LessonController::class, 'editLesson']);
Route::delete('deletelesson/{id}', [LessonController::class, 'deleteLesson']);
Route::get('/modules/{moduleId}/lessons', [LessonController::class, 'viewLessonsForModule']);
Route::get('/modules-admin/{moduleId}/lessons', [LessonController::class, 'viewLessonsForModuleAdmin']);
Route::get('/lessons/{id}', [LessonController::class,'viewLesson']);
Route::get('/lessons-admin/{id}', [LessonController::class,'viewLessonAdmin']);


// Quiz Related Routes - 
Route::post('/quizzes', [QuizController::class, 'createQuiz']);
Route::put('/quizzes/{id}', [QuizController::class, 'editQuiz']);
Route::delete('/quizzes/{id}', [QuizController::class, 'deleteQuiz']);
Route::get('/quizzes', [QuizController::class, 'viewQuizzes']);
Route::get('/lessons/{lessonId}/quizzes', [QuizController::class, 'viewQuizzesForLessonAdmin']);
Route::post('quizzes/{quizId}/questionss', [QuestionController::class, 'addquestions']);
Route::get('/quizzes-admin/{quizId}/questions-with-choices', [QuizController::class, 'getQuestionsWithChoices']);


// Task related Routes

// Route::get('/taskcount', [UserController::class, 'taskCount']);
Route::get('/taskscount', [TaskController::class, 'taskCount']);
Route::get('task-view', [TaskController::class, 'taskView']);
Route::delete('delete-task/{id}', [TaskController::class, 'DeleteTask']);
Route::put('/update-task/{id}', [TaskController::class, 'updateTask']);
Route::post('/create-task', [TaskController::class, 'createTask']);
Route::post('/assign-task/{userId}', [TaskController::class, 'assignTasks']);
Route::delete('/users/{userId}/deassign-task/{taskId}', [TaskController::class, 'deassignTask']);
Route::get('employee-tasks',[TaskController::class, 'tasksForEmployee']);
Route::put('/tasks/{taskId}/update-to-in-progress', [TaskController::class,'updateTaskToInProgress']);
Route::put('/tasks/{taskId}/update-to-completed', [TaskController::class,'updateTaskToCompleted']);
Route::get('/user/assigned-tasks', [UserController::class, 'getAssignTasks']);
Route::get('task-not-assigned-view', [TaskController::class, 'unassignedTasks']);
Route::get('/assigned-task-count-for-admin',[TaskController::class,'getAssignedTasksCountAdmin']);
Route::get('/assigned-task-count-for-user',[TaskController::class,'getAssignedTasksCountUser']);



// All Count Related Routes - 

Route::get('/usercount', [UserController::class, 'userCount']);
Route::get('/coursecount', [NewCourseController::class, 'courseCount']);
Route::get('modulecount', [ModuleController::class, 'moduleCount']);
Route::get('lessoncount', [LessonController::class, 'lessionCount']);
Route::get('/assigned-course-count-for-admin',[NewCourseController::class,'getAssignedCoursesCountAdmin']);

// Routes for Files - 
Route::post('/upload-files', [FileUploadController::class, 'uploadFile']);
Route::get('/getfiles', [FileUploadController::class, 'getFiles']);


    // assigned courses and tasks api - 

Route::get('/users-with-assigned-courses', [UserController::class, 'getUsersWithAssignedCourses']);
Route::get('/employees-with-assigned-tasks', [UserController::class, 'getUsersWithAssignedTasks']);


    

 // Email Templates Related Routes - 

Route::post('/email-templates', [EmailTemplateController::class, 'store']); 
Route::get('/email-templates/{template_category_id?}', [EmailTemplateController::class, 'index']);
Route::get('/email-templatess/{id}', [EmailTemplateController::class, 'singletemplate']);
Route::put('/email-templates/{id}', [EmailTemplateController::class, 'update']);
Route::delete('/email-templates/{id}', [EmailTemplateController::class, 'destroy']);
Route::get('email-track-count',[EmailTrackingController::class,'emailTrackCount']);
Route::get('/email-sent-track',[EmailSentTrackController::class,'emailSentTrack']);
    // Routes for Template Categories - 

Route::apiResource('template-categories', TemplateCategoryController::class);

// Landing Page Related Routes - 

Route::get('landing-pages/{id?}', [LandingPageController::class, 'index']);
Route::get('landing-page/{id}', [LandingPageController::class,'singlePage']);
Route::post('/landing-pages', [LandingPageController::class, 'store']);
Route::put('/landing-pages/{id}', [LandingPageController::class, 'update']);
Route::delete('/landing-pages/{id}', [LandingPageController::class, 'destroy']);

//Landing Pages Category Routes - 

Route::apiResource('landing-categories', LandingPageCategoryController::class);

//Quick Links - 

Route::get('/quick-links', [QuickLinkController::class, 'index']);
Route::post('/quick-links', [QuickLinkController::class, 'store']);
Route::put('/quick-links/{quickLink}', [QuickLinkController::class, 'update']);
Route::delete('/quick-links', [QuickLinkController::class, 'destroy']);

// manage quick links

Route::get('/manage-quick-links', [ManageQuickLinkController::class, 'index']);
Route::post('/manage-quick-links', [ManageQuickLinkController::class, 'store']);
Route::get('/manage-quick-links/{id}', [ManageQuickLinkController::class, 'show']);
Route::put('/manage-quick-links/{id}', [ManageQuickLinkController::class, 'update']);
Route::delete('/manage-quick-links/{adminid}/{linkid}', [ManageQuickLinkController::class, 'destroy']);
/** San */
Route::get('/users-with-not-started-courses', [UserController::class, 'getUsersWithNotStartedCourses']);
Route::get('/users-with-not-started-tasks', [UserController::class, 'getUsersWithNotStartedTasks']);
Route::get('/employee-progress', [UserController::class, 'getEmployeeProgress']);

// Two Factor Authentication - 
Route::post('/request-otp', [TwoFAController::class, 'requestOtp']);
Route::post('/verify-otp', [TwoFAController::class, 'verifyOtp']);


// Route for tracking email phishing section - 

Route::get('/tracking/report/{eventType}/{userId}/{templateId}', [EmailTrackingController::class, 'getReport']);

// Route for sending email to multiple users - 

Route::post('/send-email-multiple/{templateId}', [EmailTemplateController::class, 'sendEmailToMultipleUsers']);

// Route for sending test email - 

Route::post('/send-email/{templateId}/{userId}', [EmailTemplateController::class, 'sendEmail']);


// Fetching Admin data by super admin - 

Route::get('/fetch-admin-data/{adminId}', [UserController::class, 'fetchAdminData']);

// Send Notifications to employees for task completion - 

Route::post('/send-notification/{userId}', [TaskController::class, 'sendNotification']);

// Send Notification to employees for course completion

Route::post('/send-notification/{userId}/course', [TaskController::class, 'sendCourseNotification']);


// Get all Task counts with status for admin 

Route::get('/get-tasks-status', [TaskController::class, 'getTaskCounts']);

// Get all Task counts with status for user 

Route::get('/get-tasks-status-user', [TaskController::class, 'getTaskCountsUser']);

// Get all task status for a particular admin
Route::get('/get-user-tasks/{userId}', [TaskController::class, 'getUserTasks']);

// Overall email phishing data - 

Route::get('/tracking/events', [EmailTrackingController::class, 'getTrackingEvents']);

    // PolicyController routes
    Route::resource('policies', PolicyController::class);
    
    Route::post('accounts/{accountId}/assign-admin', [AccountAdminController::class, 'assignAdmin']);
    Route::post('accounts/{accountId}/remove-admin', [AccountAdminController::class, 'removeAdmin']);
    Route::get('/get-assigned-admins', [AccountAdminController::class, 'getAssignedAdmins']);
    Route::get('/get-assigned-accounts/{id}', [AccountAdminController::class, 'getTotalAssignedAccounts']);

});


Route::post('forgetpassword', [AuthController::class, 'forgetPassword']);
Route::post('/signup', [AuthController::class, 'signup']);
Route::post('/login', [AuthController::class, 'login']);
Route::post('/verify-login-otp', [AuthController::class, 'verifyLoginOtp']);

// Routes for Files

Route::put('/editfiles/{id}',[FileUploadController::class,'editFile']);
Route::delete('/deletefiles/{id}',[FileUploadController::class,'deleteFile']);

//Route for Google Authentication 

Route::post('/google-auth', [AuthController::class, 'googleLogin']);
Route::post('/send-otps', [ForgotPasswordController::class, 'sendOtp']);
Route::post('/verify-otps', [ForgotPasswordController::class, 'verifyOtpAndReset']);


Route::apiResource('quizzes.questions', QuestionController::class)->only(['store']);
Route::apiResource('questions.choices', ChoiceController::class)->only(['store']);
Route::get('/questions/{questionId}/correct-answer', [ChoiceController::class, 'getCorrectAnswer']);
Route::get('/users/{userId}/calculate-score', [ChoiceController::class, 'calculateUserScore']);
Route::get('/user/{userId}/lesson/{lessonId}/quizzes', [QuizController::class, 'viewQuizzesForLessonwithstatus']);
Route::get('/quizzes/{id}', [QuizController::class, 'viewQuiz']);
Route::delete('/quizzes/{quizId}/questions/{questionId}', [QuestionController::class, 'deleteQuestion']);
Route::put('/quizzes/{quizId}/questions/{questionId}', [QuestionController::class, 'updateQuestion']);
Route::get('/users/{userId}/completed-courses', [NewCourseController::class, 'completedCourses']);
Route::get('/users/{userId}/completed-tasks', [NewCourseController::class, 'completedTasks']);
Route::post('/send-emails/{templateId}', [EmailTemplateController::class, 'sendEmails']);
Route::get('/email/open/{templateId}/{email}/{userId}/{url}', [EmailTrackingController::class, 'trackOpen'])->name('email.open');
Route::post('/email/click', [EmailTrackingController::class, 'trackClick']);

Route::get('landing-page-redirect/{id}', [LandingPageController::class,'singlePage']);



